#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "utilities.h"


/***************************************************************/
/*                                                             */
/* String/character manipulation routines                      */
/*                                                             */
/***************************************************************/


/* take string as an argument and return the substing starting */
/* at character "begin" and ending at character "end"          */
char *str_substring(char *sub, char *string, int begin, int end) {
  
  char *sss;
  int len = strlen(string);
  int i, j;

  /* allocate memory */
  sss = (char *) mem_malloc_safe((strlen(string) + 1)*sizeof(char));
  strcpy(sss,string);

  if ((len < (end - 1)) ||
      (begin > end) ||
      (begin < 0)) {
    strcpy(sub,"");
  } else {
    for (i = begin, j = 0; i <= end; i++, j++) {
      sub[j] = sss[i];
    }
    if (sub[j-1] != '\0') sub[j] = '\0';
  }
    
  return sub;

}

/* return the first position of character "character" in string "string" */
/* MODIFY: to start searching after character n                          */
int str_position(char *string, char character) {
  
  char *sss;
  int len = strlen(string), i = -1;
  int found = -1;

  /* allocate memory */
  sss = (char *) mem_malloc_safe((strlen(string) + 1)*sizeof(char));
  strcpy(sss,string);

  while ((i++ < len) && (found < 0)) {
    if (sss[i] == character) found = i;
  }

  return found;

}

/* return the last position of character "character" in string "string" */
/* MODIFY: to start searching before character n                        */
int str_positionr(char *string, char character) {
  
  char *sss;
  int len = strlen(string), i = len;
  int found = -1;

  /* allocate memory */
  sss = (char *) mem_malloc_safe((strlen(string) + 1)*sizeof(char));
  strcpy(sss,string);

  while ((i-- >0) && (found < 0)) {
    if (sss[i] == character) found = i;
  }

  return found;

}

char *str_remove_char(char *_new_, char *old, char c) {

  int i, j, n = strlen(old);

  for (i = 0, j = 0; i <= n; i++) 
    when(old[i] != c,
	   _new_[j++] = old[i];)

  return _new_;
      
}

char *str_remove_digits(char *_new_, char *old) {

  int i, j, n = strlen(old);
  
  for (i = 0, j = 0; i <= n; i++) 
    when(!(isdigit(old[i])),
	   _new_[j++] = old[i];)

  return _new_;
      
}

char *str_upcase(char *_new_, char *old) {

  int i, n = strlen(old);

  for (i = 0; i < n; i++) {
    if (old[i] >= 'a' && old[i] <= 'z')
      _new_[i] = old[i] - 32;
    else 
      _new_[i] = old[i];
  }

  return _new_;

}

char *str_downcase(char *_new_, char *old) {

  int i, n = strlen(old);

  for (i = 0; i < n; i++) {
    if (old[i] >= 'A' && old[i] <= 'Z')
      _new_[i] = old[i] + 32;
    else 
      _new_[i] = old[i];
  }

  return _new_;

}

/***************************************************************/
/*                                                             */
/* Messaging routines                                          */
/*                                                             */
/***************************************************************/

int _msg_verbose_ = 99;         /* verbosity level: 
				   0 = completely quiet 
				   1 = only the important messages
				   2 = report everything       */
msg_report *_msg_files_;

int msg_message(int verb, const char *fmt, ...) {

  va_list ap;
  msg_report *files;

  /* initialize _msg_files_ if it is empty */
  if (_msg_files_ == NULL) msg_initialize();

  if (_msg_verbose_ >= verb || _msg_verbose_ == MSG_ALLWAYS) {
    files = _msg_files_;
    va_start(ap,fmt);           /* setup variable argument list */
    
    /* 1: print to first file */
    vfprintf(files->file,fmt,ap);
    fflush(files->file);
  
    /* 2: print to other possible files */
    while (files->next != NULL) {
      files = files->next;
      vfprintf(files->file,fmt,ap);
      fflush(files->file);
    }

    va_end(ap);                 /* close variable argument list    */

  }

  return 0;

}

int msg_error(const char *fmt, ...) {

  va_list ap;
  msg_report *files;

  files = _msg_files_;
  va_start(ap,fmt);           /* setup variable argument list */
  
  /* 1: print to first file */
  vfprintf(files->file,fmt,ap);
  fflush(files->file);
  
  /* 2: print to other possible files */
  while (files->next != NULL) {
    files = files->next;
    vfprintf(files->file,fmt,ap);
    fflush(files->file);
  }

  va_end(ap);                 /* close variable argument list    */

  return 0;

}

void msg_exit(int status, const char *fmt, ...) {

  va_list ap;
  msg_report *files;

  /* initialize _msg_files_ if it is empty */
  if (_msg_files_ == NULL) msg_initialize();

  files = _msg_files_;
  va_start(ap,fmt);           /* setup variable argument list */
  
  /* 1: print to first file */
  vfprintf(files->file,fmt,ap);
  fflush(files->file);
  
  /* 2: print to other possible files */
  while (files->next != NULL) {
    files = files->next;
    vfprintf(files->file,fmt,ap);
    fflush(files->file);
  }

  va_end(ap);                 /* close variable argument list    */

  msg_close();                /* close all open message files    */

  exit(status);               /* Exit the program                */

}

int msg_set_verbose(int verbose) {
  /* initialize or change the verbosity level of the messaging */
  /* routines.                                                 */

  int old_verbose = _msg_verbose_;

  /* initialize _msg_files_ if it is empty */
  if (_msg_files_ == NULL) msg_initialize();

  if ((verbose >= 0) &&
      (verbose <= MSG_MAX_VERBOSE)) {
    _msg_verbose_ = verbose;
    msg_message(1,">>> Verbosity level changed\n");
    msg_message(2,"    from %d to %d\n", old_verbose, verbose);
  } else {
    msg_error("*** Failed to change verbosity level: must receive a value between 0 and %d\n", MSG_MAX_VERBOSE);
    return -1;
  }

  return verbose;

}

int msg_initialize() {

  /* Set output to standard output as the default */
  char stdoutname[] = "stdout";

  if (_msg_files_ != NULL) msg_close();

  _msg_files_ = (msg_report *) mem_malloc_safe(sizeof(msg_report));
  _msg_files_->file = stdout;
  _msg_files_->filename = 
    (char *) mem_malloc_safe((strlen(stdoutname)+1)*sizeof(char));
  strcpy(_msg_files_->filename,stdoutname);

  return 0;

}

void msg_fprintf(const char *fmt, ...) {

  va_list ap;                /* variable argument list structure */
  msg_report *files;          /* temporary pointer to MESS_REPORT */

  /* initialize _msg_files_ if it is empty */
  if (_msg_files_ == NULL) msg_initialize();

  files = _msg_files_;      /* copy output structure            */

  va_start(ap, fmt);         /* Point ap to first message.       */

  /* 1: print to first file */
  vfprintf(files->file,fmt,ap);
  fflush(files->file);
  
  /* 2: print to other possible files */
  while (files->next != NULL) {
    files = files->next;
    vfprintf(files->file,fmt,ap);
    fflush(files->file);
  }

  va_end(ap);                 /* close variable argument list    */

}

int msg_add_file(const char *filename, const char *mode) {
  /* append another output file to the list of files     */
  /* which are receiving the message output              */

  FILE *file;
  msg_report *files;

  /* initialize _msg_files_ if it is empty */
  if (_msg_files_ == NULL) msg_initialize();


  /* filter out the standard output files stdout and stderr */
  if (!(strcmp(filename,"stdout"))) {
    file = stdout;
  } else if (!(strcmp(filename,"stderr"))) {
    file = stderr;
  } else {
    /* open a new or exisiting file */
    if (!(!(strcmp(mode,"r"))  || !(strcmp(mode,"w"))  || 
	  !(strcmp(mode,"a"))  || !(strcmp(mode,"r+")) || 
	  !(strcmp(mode,"w+")) || !(strcmp(mode,"a+"))))
      file = fopen(filename,"w");
    else
      file = fopen(filename,mode);
  }

  
  files = _msg_files_;

  if (file != NULL) {

    while (files->next != NULL) files = files->next;
    files->next = (msg_report *) mem_malloc_safe(sizeof(msg_report));
    files = files->next;

    /* assign filename and file pointer to MSG_REPORT structure */
    files->filename = (char *) mem_malloc_safe((strlen(filename)+1)*sizeof(char));
    strcpy(files->filename,filename);
    files->file = file;

  } else {
    msg_error("*** Failed to add message output file: receive an empty pointer\n");
    return -1;
  }

  return 0;

}

int msg_remove_file(const char *filename) {
  /* remove an output file from the list of files        */
  /* which are receiving the message output              */

  msg_report *previous, *current, *next;
  int scmp;

  /* initialize _msg_files_ if it is empty, but if empty 
     it is impossible to remove anything from it. So bail 
     out.                                                 */
  if (_msg_files_ == NULL) {
    msg_initialize();
    return -1;
  }
  
  previous = NULL;
  current = _msg_files_;
  next = _msg_files_->next;
  
  while ((scmp = strcmp(current->filename,filename)) && next != NULL) {
      previous = current;
      current = next;
      next = current->next;
  }

  if (previous == NULL) {
    /* first file in list must be removed */
    msg_fclose(current->file, current->filename); /* close file  */
    free(current);                    /* free memory */
                                      /* direct pointer to next file */
    _msg_files_ = (msg_report *) mem_malloc_safe(sizeof(msg_report));
    _msg_files_ = next;
  } else if (next == NULL && scmp == 0) {
    /* remove the last file from the list */
    msg_fclose(current->file, current->filename);
    free(current);
    previous->next = NULL;
  } else if (next == NULL) {
    /* did not find file to be removed */
    msg_error("*** Could not find file (%s) in the output file list. Leaving list unchanged\n", filename);
    return -1;
  } else {
    msg_fclose(current->file, current->filename);
    free(current);
    previous->next = next;
  }

  return 0;

}

void msg_fclose(FILE *file, const char *filename) {
  
  if (!(!(strcmp(filename,"stdout")) || !(strcmp(filename,"stderr"))))
    fclose(file);

}

void msg_close() {

  /* free memory occupied but the MESS_REPORT list and close all */
  /* files that are currenlty open                               */

  msg_report *files;

  files = _msg_files_;

  while (files != NULL) {
    _msg_files_ = files;
    fclose(files->file);       /* close the file       */
    files = files->next;       /* move on to next file */
    free(_msg_files_)  ;       /* free memory current 
				   structure            */
  }

}

/***************************************************************/
/*                                                             */
/* Time and Date routines                                      */
/*                                                             */
/***************************************************************/
const int monthn[13] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 
			273, 304, 334, 365};
const int month[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

const int yzero = 2000;

date *time_adddays(date *date, int n) {

  date->n += n;
  time_getdatefromn(date);

  date->leap = time_isleap(date->yea);
  date->dow = time_dow(date);
  date->bus = time_isbusinessday(date); 
  

  return date;

}

date *time_addweeks(date *date, int n) {

  date->n += 7*n;
  time_getdatefromn(date);

  date->leap = time_isleap(date->yea);
  date->dow = time_dow(date);
  date->bus = time_isbusinessday(date); 
  

  return date;

}

date *time_addmonths(date *date, int n) {

  date->mon += n;
  if (date->mon > 0) {
    while (date->mon > 12) {
      date->yea +=  1;
      date->mon -= 12;
    }
  } else {
    while (date->mon <= 0) {
      date->yea -=  1;
      date->mon += 12;
    }
  }
  if (date->day > time_daysinm(date->mon,date->yea)) 
    date->day = time_daysinm(date->mon,date->yea);
  date->n = time_getndays(date);

  date->leap = time_isleap(date->yea);
  date->dow = time_dow(date);
  date->bus = time_isbusinessday(date); 
  

  return date;

}

date *time_addyears(date *date, int n) {

  date->yea += n;
  if (date->mon == 2 && !(time_isleap(date->yea)) && date->day == 29)
    date->day -= 1;
  date->n = time_getndays(date);

  date->leap = time_isleap(date->yea);
  date->dow = time_dow(date);
  date->bus = time_isbusinessday(date); 
  

  return date;

}

date *time_addbusinessdays(date *date, int n) {

  if (n >= 0) {
    while (n > 0) if (time_isbusinessday(time_adddays(date,1))) n--;
  } else {
    while (n < 0) if (time_isbusinessday(time_adddays(date,-1))) n++;  
  }

  return date;

}

int time_getndays(date *date) {

  int i, n = 0;
  int dy = (date->yea - yzero);    /* difference in years */

  if (dy >= 0) {
    for (i = 0; i < dy; i++) 
      n += (time_isleap(yzero + i) == 1) ? 366 : 365; 
    n += time_daysuntilm(date->mon-1,yzero+i);
    n += date->day;
  } else {
    for (i = -1; i >= dy; i--) {
      n -= (time_isleap(yzero + i) == 1) ? 366 : 365; 
    }
    i++;
    n += time_daysuntilm(date->mon-1,yzero+i);
    n += date->day;

  }
  
  return n;

}


int time_getdatefromn(date *date) {

  int n = date->n;
  int m, y, days;
  
  date->day = 0;
  date->mon = 0;
  date->yea = 0;


  if (n >= 1) {
    y = yzero;
    days = time_daysperyear(y);
    while ((n - days) > 0) {
      n -= days;
      y++;
      days = time_daysperyear(y);
    }
    date->yea = y;
    
    m = 1;
    while (n > time_daysuntilm(m,y))
      m++;
    n -= time_daysuntilm(m-1,y);
    date->mon = m;
    date->day = n;

  } else {
    y = yzero - 1 ;
    days = time_daysperyear(y);
    while (n + days <= 0) {
      n += days;
      y--;
      days = time_daysperyear(y);
    }
    date->yea = y;

    n += time_daysperyear(y);

    m = 1;
    while (n > time_daysuntilm(m,y))
      m++;
    n -= time_daysuntilm(m-1,y);
    date->mon = m;
    date->day = n;
    
  }

  return 0;

}

int time_daysuntilm(int mon, int yea) {
  
  int d = monthn[mon];

  d += (mon >= 2 && time_isleap(yea)) ? 1 : 0;

  return d;

}

int time_daysinm(int mon, int yea) {
  
  int d = month[mon];

  d += (time_isleap(yea)) ? 1 : 0;

  return d;

}

int time_daysperyear(int yea) {

  return (time_isleap(yea) == 1) ? 366 : 365;

}



int time_dow(date *date) {

  int dow = date->n%7;

  if (dow < 0) dow += 7;

  return dow;

}

int time_isleap(int year) {

  if ((year%4 == 0 && year%100 != 0) || year%400 == 0) {
    return 1;
  } else
    return 0;

}

int time_isbusinessday(date *date) {

  if (date->dow == SATURDAY || date->dow == SUNDAY) 
    return 0;
  else
    return 1;

}

void time_setdate(date *date, int mon, int day, int yea) {

  int dum = 0;

  date->day = day;
  date->mon = mon;
  date->yea = yea;
  date->leap = time_isleap(yea);
  /* check validity of input */
  if (mon > 12) {
    msg_message(0,"Month must be smaller than 12 (December), received %d\n",
		mon);
    mon = 12;
    date->mon = mon;
  }

  dum += month[mon]; dum += (mon == 2 && date->leap) ? 1 : 0;
  if (day > dum) {
    msg_message(0,"Day must be smaller than %d, received %d\n",
		dum, day);
    day = dum;
    date->day = day;
  }
  /* end check */
  date->n = time_getndays(date);
  date->dow = time_dow(date);
  date->bus = time_isbusinessday(date); 
  date->sec = 0;
  date->min = 0;
  date->hou = 0;
  date->dst = 0;

}

void time_printdate(date *date) {
  
  printf("\nDate structure \n");
  printf(" Day   : %d\n", date->day); 
  printf(" Month : %d\n", date->mon); 
  printf(" Year  : %d\n", date->yea); 
  printf(" #days : %d\n", date->n); 
  time_printdow(date);
  printf(" Bus.  : %d\n", date->bus);
  printf(" Leap  : %d\n", date->leap); 
  printf(" DLST  : %d\n", date->dst); 
  printf(" Sec   : %d\n", date->sec); 
  printf(" Min   : %d\n", date->min); 
  printf(" Hou   : %d\n", date->hou); 

} 


void time_printdow(date *date) {

  char day[9] = "";

  switch (date->dow) {
  case 0:  strcpy(day,"Friday");    break;
  case 1:  strcpy(day,"Saturday");  break;
  case 2:  strcpy(day,"Sunday");    break;
  case 3:  strcpy(day,"Monday");    break;
  case 4:  strcpy(day,"Tuesday");   break;
  case 5:  strcpy(day,"Wednesday"); break;
  case 6:  strcpy(day,"Thursday");  break;
  default: strcpy(day,"Unknown");   break;
  }

  printf(" DoW   : %s\n", day);

}

char *time_get_ut_string(char *stime) {


  /* '2002-03-04T21:04:31' */

  time_t t1;
  struct tm *t3;
  
  time(&t1);

  t3 = mem_malloc_safe(sizeof(struct tm));
  t3 = gmtime(&t1); 

  sprintf(stime,"%04d-%02d-%02dT%02d:%02d:%02d",
	  t3->tm_year+1900,t3->tm_mon+1,t3->tm_mday,
	  t3->tm_hour,t3->tm_min,t3->tm_sec);

  free(t3);

  return stime;

}


/***************************************************************/
/*                                                             */
/* Memory management routines                                  */
/*                                                             */
/***************************************************************/


/* provide a safe way of allocating memory. The code will abort */
/* if the memory allocation is not successful.                  */
/*                                                              */
void * mem_malloc_or_die(size_t size, char *routine, int line) {

  void *ptr;

  ptr = malloc(size);
  if (ptr == NULL) {
    mem_malloc_failed_message(routine, line);
  }
  return ptr;
}

void mem_malloc_failed_message(const char *routine, const int line) {

  printf("Memory allocation error in: %s (at line: %d)\n",routine, line);
  printf("\n Aborting. \n");
  fflush(stdout);
  abort();
}

