#include <stdlib.h>
#include <stdio.h>

#include <fitsio.h>

#define NULL_VALUE 0.0

void fits_printerror(char *process, int status);

void fits_spec_write_cocorow(fitsfile *fptr, int iRow, int Z, int rmJ, 
			     int Max_Real, int Max_Pseudo, 
			     int N_Real, double *E_Real, double *Real, 
			     int N_Pseudo, double *E_Pseudo, double *Pseudo){

  int colnum, iBin;
  long firstrow, firstelem, nelements;
  int status=0;
  float *E_Real_fl,  *Real_fl;
  float *E_Pseudo_fl, *Pseudo_fl;

  E_Real_fl  = (float *) malloc(Max_Real*sizeof(float));
  Real_fl    = (float *) malloc(Max_Real*sizeof(float));
  E_Pseudo_fl= (float *) malloc(Max_Pseudo*sizeof(float));
  Pseudo_fl  = (float *) malloc(Max_Pseudo*sizeof(float));

  for (iBin=0; iBin<Max_Real; iBin++) {
    if (iBin < N_Real) {
      E_Real_fl[iBin] = (float) E_Real[iBin];
      Real_fl[iBin]   = (float) Real[iBin];
    } else {
      E_Real_fl[iBin] = (float) NULL_VALUE;
      Real_fl[iBin]   = (float) NULL_VALUE;
    }
  }
  for (iBin=0; iBin<Max_Pseudo; iBin++) {
    if (iBin < N_Pseudo) {
      E_Pseudo_fl[iBin] = (float) E_Pseudo[iBin];
      Pseudo_fl[iBin]   = (float) Pseudo[iBin];
    } else {
      E_Pseudo_fl[iBin] = (float) NULL_VALUE;
      Pseudo_fl[iBin]   = (float) NULL_VALUE;
    }
  }

  /* Write one row to the FITS file. */

  firstrow = iRow;
  firstelem = 1;
  nelements = 1;

  colnum = 1;
  if (fits_write_col(fptr, TINT, colnum, firstrow, firstelem,
		     nelements, &Z, &status)){
      fits_printerror("writing element",status);
  }

  colnum = 2;
  if (fits_write_col(fptr, TINT, colnum, firstrow, firstelem,
		     nelements, &rmJ, &status)){
      fits_printerror("writing ion",status);
  }

  colnum = 3;
  if (fits_write_col(fptr, TINT, colnum, firstrow, firstelem,
		     nelements, &N_Real, &status)){
      fits_printerror("writing number of real values",status);
  }
  
  colnum = 4;
  nelements = Max_Real;
  if (fits_write_col(fptr, TFLOAT, colnum, firstrow, firstelem,
		     nelements, E_Real_fl, &status)){
    fits_printerror("writing real continuum energies",status);
  }

  colnum = 5;
  if (fits_write_col(fptr, TFLOAT, colnum, firstrow, firstelem,
		     nelements, Real_fl, &status)){
      fits_printerror("writing real continuum emissivities",status);
  }

  colnum = 7;
  nelements = 1;
  if (fits_write_col(fptr, TINT, colnum, firstrow, firstelem,
		     nelements, &N_Pseudo, &status)){
    fits_printerror("writing number of pseudo values",status);
  }

  colnum = 8;
  nelements = Max_Pseudo;
  if (fits_write_col(fptr, TFLOAT, colnum, firstrow, firstelem,
		     nelements, E_Pseudo_fl, &status)){
      fits_printerror("writing pseudo-continuum energies",status);
  }
  colnum = 9;
  if (fits_write_col(fptr, TFLOAT, colnum, firstrow, firstelem,
		     nelements, Pseudo_fl, &status)){
      fits_printerror("writing pseudo-continuum emissivities",status);
  }

  free(E_Real_fl);
  free(Real_fl);
  free(E_Pseudo_fl);
  free(Pseudo_fl);
  
}
