#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "fitsspec.h"

int main(int argc, char *argv[]) {

  fitsfile *fptr_line, *fptr_coco;
  int N_Z, iZ, *list_Z, NRow, iRow;
  int NLine, iLine, NBin, iBin;
  double T, n_e;
  double *lambda, *epsilon, *cont_arr, *pseu_arr, *E_low, *E_high;
  int *el_Z, *rmJ;

  fits_spec_init("Test", 0, 0, &fptr_line);
  fits_spec_init("Test", 2, 0, &fptr_coco);

  NRow = 2;
  N_Z = 3;
  list_Z = (int *) malloc(N_Z*sizeof(int));
  for (iZ=0;iZ<N_Z;iZ++) list_Z[iZ] = iZ + 6;  /* C, N, O */

  NLine = 10;
  lambda  = (double *) malloc(NLine*sizeof(double));
  epsilon = (double *) malloc(NLine*sizeof(double));
  el_Z    = (int *) malloc(NLine*sizeof(int));
  rmJ     = (int *) malloc(NLine*sizeof(int));

  for (iLine=0;iLine<NLine;iLine++) {
    lambda[iLine] = 10.0*(1 + iLine);
    epsilon[iLine]= 1.e-15*(1 + iLine);
    el_Z[iLine]  = list_Z[iLine - N_Z*(iLine/N_Z)];
    rmJ[iLine]   = list_Z[iLine - N_Z*(iLine/N_Z)] - 2;
  }

  NBin = 10000;
  E_low    = (double *) malloc(NBin*N_Z*sizeof(double));
  E_high   = (double *) malloc(NBin*N_Z*sizeof(double));
  cont_arr = (double *) malloc(NBin*N_Z*sizeof(double));
  pseu_arr = (double *) malloc(NBin*N_Z*sizeof(double));
  for (iZ=0;iZ<N_Z;iZ++) {
    for (iBin=0;iBin<NBin;iBin++) {
      E_low[iZ + N_Z*iBin]  = 0.1 + 0.001*iBin;
      E_high[iZ + N_Z*iBin] = 0.1 + 0.001*(iBin+1);
      cont_arr[iZ + N_Z*iBin] = 1.e-20*(iBin - 1000*(iBin/1000) + iZ*500);
      pseu_arr[iZ + N_Z*iBin] = 1.e-19*(iBin -  333*(iBin/333)  + iZ*100);
    }
  }

  for (iRow=0;iRow<NRow;iRow++) {
    T = 1.e6*(1 +iRow);
    n_e = 1.e10*(1 + iRow);
    fits_spec_write_lines(fptr_line, T, n_e, NLine, lambda, NULL, epsilon, 
			  NULL, el_Z, rmJ, NULL, NULL);

    fits_spec_write_coco(fptr_coco, N_Z, list_Z, NULL, T, n_e, NBin, E_low, 
			 E_high, pseu_arr, cont_arr);
  }

  fits_spec_close(fptr_line);
  fits_spec_close(fptr_coco);
}
