/* Subroutine exint1(x,jump),
   translated from Fortran.
   This subroutine can be called by an external main program, such
   as call_exint1.c.

   jump = 1: exint1 = E1(x);
   jump = 2: exint1 = exp(x) * E1(x);
   jump = 3: exint1 = x * exp(x) * E1(x);
   Returns a double precision floating point value.
*/

#include <math.h>

double exint1(double x, int jump) {

  double EI_1;
  double EI_2;
  double EI_3;
  double x2;
  double x3;
  double x4;
  static double a[] = {
    7.122452e-07, 1.766345e-06, 2.928433e-05, .0002335379, .001664156,
    .01041576, .05555682, .2500001, .9999999, .57721566490153};
  static double b[] = {
    8.5733287401, 18.059016973, 8.6347608925, .2677737343, 9.5733223454,
    25.6329561486, 21.0996530827, 3.9584969228};

  if (x==0.) {
    return(-1.);
  } else {
    if (x <= 1.0){
      EI_1 = ((((((((a[0] * x - a[1]) * x + a[2]) * x
		   - a[3]) * x + a[4]) * x - a[5]) * x
		+ a[6]) * x - a[7]) * x + a[8]) * x
	- log(x) - a[9];
      EI_2 = exp(x) * EI_1;
      EI_3 = x * EI_2;
    } else {
      x2 = pow(x,2.);
      x3 = pow(x,3.);
      x4 = pow(x,4.);
      
      EI_3 = (x4 + b[0] * x3 + b[1] * x2 
	      + b[2] * x + b[3]) / 
	(x4 + b[4] * x3 + b[5] *x2
	 + b[6] * x + b[7]);
      
      EI_1 = EI_3 / (x * exp(x));
      EI_2 = EI_3 / x;
    }
  }
  
  /* In K&R C, the switch argument has to be of type
     int.Can terminate with return or break.
     */
  
  switch (jump) {
  case 1:
    return(EI_1);
  case 2:
    return(EI_2);
  case 3:
    return(EI_3);
  default:
    return(-1.);
  }
  
}
