#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "messages.h"

#ifndef KBOLTZ
#define KBOLTZ 8.617385e-8  /* kB = [keV/K] */
#endif

double exint_n(double x, double E1x_in, int n);

double calc_kato(int coll_type, double par[], int Z, double Te) {

  /* This fit comes from Kato & Nakazaki, 1989, Atomic Data and Nuclear 
     Data Tables 42, 2, 313 */
  
  double result=0.0;
  double dE, kT;
  double A, B, C, D, E, P, Q, X1;
  double y;
  double E1y, E1Xy;
  double ups_nr, ups_r;
  double term1, term2, term3;

  dE = par[0];  /* in keV */
  A  = par[1];
  B  = par[2];
  C  = par[3];
  D  = par[4];
  E  = par[5];
  P  = par[6];
  Q  = par[7];
  X1 = par[8];
  
  kT = KBOLTZ*Te;
  y = dE/kT;

  switch (coll_type) {
  case (1):  /* Simple case (eq 6, in above reference) */
    E1y = exint_n(y,-1.,1);
    term1 = A/y + C + (D/2)*(1-y);
    term2 = B - C*y + (D/2)*y*y + E/y;
    result = y*(term1 + exp(y)*E1y*term2);

    break;
  case (2):  /* More complex case (eq 10-12 in above reference) */
    E1Xy = exint_n(X1*y,-1.,1);
    term3 = exp(y*(1-X1));
    
    term1 = A/y + C/X1 + (D/2)*(1/(X1*X1) - y/X1) + (E/y)*log(X1);
    term2 = exp(y*X1)*E1Xy*(B - C*y + D*y*y/2 + E/y);

    ups_nr = y*term3*( term1 + term2 );
    ups_r = P*((1 + 1/y) - term3*(X1 + 1/y)) + Q*(1-term3);

    result = ups_nr + ups_r;
    break;
  }

  return result;
}

