#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "apec.h"
#include "datatypes.h"
#include "messages.h"

void create_chksum_arrays(int N_Elements, int *list_Z, 
			  ElementP *Atomic_data, struct CHKSUM **chksum_top) {

  /* ******************************************************************* */
  /* Routine creates a linked list of structures, returning the FITS     */
  /* checksum values of all the input datafiles.  This list is then      */
  /* written to the output FITS files to provide a trace to the original */
  /* data used.                                                          */
  /* ******************************************************************* */

  int iEl, rmJ, Z;
  struct ELEMENT_ION_STATE *state; 
  struct CHKSUM *ptr, *tmp;

  ptr = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
  *chksum_top = ptr;
  ptr->next = NULL;

  for (iEl=0;iEl<N_Elements;iEl++) {
    state = Atomic_data[iEl];	  
    Z = list_Z[iEl];
    for (rmJ=1;rmJ<Z+1;rmJ++) {
      if (state[rmJ-1].ion_inf->cs_ionrec != 0) {
	ptr->type = DATATYPE_IONREC;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_ionrec);
	sprintf(ptr->keyname, "CSIR%02i%02i", ptr->Z, ptr->rmJ);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
      if (state[rmJ-1].ion_inf->cs_elvls != 0) {
	ptr->type = DATATYPE_LEVEL;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyname, "CSLV%02i%02i", Z, rmJ);
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_elvls);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
      if (state[rmJ-1].ion_inf->cs_lines != 0) {
	ptr->type = DATATYPE_RADIATION;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyname, "CSLA%02i%02i", Z, rmJ);
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_lines);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
      if (state[rmJ-1].ion_inf->cs_ecoll != 0) {
	ptr->type = DATATYPE_E_COLLISION;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyname, "CSEC%02i%02i", Z, rmJ);
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_ecoll);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
      if (state[rmJ-1].ion_inf->cs_pcoll != 0) {
	ptr->type = DATATYPE_P_COLLISION;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyname, "CSPC%02i%02i", Z, rmJ);
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_pcoll);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
      if (state[rmJ-1].ion_inf->cs_drsat != 0) {
	ptr->type = DATATYPE_SATELLITE;
	ptr->Z = Z;
	ptr->rmJ = rmJ;
	sprintf(ptr->keyname, "CSDR%02i%02i", Z, rmJ);
	sprintf(ptr->keyval, "%-lu", state[rmJ-1].ion_inf->cs_drsat);
	ptr->next = (struct CHKSUM *) malloc_safe(sizeof(struct CHKSUM));
	ptr = ptr->next;
	ptr->next = NULL;
      }
    }
  }
  /* Now, there is a spare ptr at the end of the linked list.  Delete it. */

  tmp = *chksum_top;
  if (tmp->next == NULL) {
    *chksum_top = NULL; 
  } else {
    while (tmp->next != ptr) tmp = tmp->next;
    tmp->next = NULL;
    free(ptr);
  }
}
