#include <stdlib.h>
#ifdef sparc
#include <ieeefp.h>
#endif

#include "apec.h"

void create_spectral_bins(int BinType, int Nbin, 
			  double BinMin, double BinMaX, 
			  int N_Elements, int list_Z[], 
			  struct BIN_SPEC *spectrum);
void get_atomic_data(char *filemap_file, int require_checksum, 
		     int N_Elements, int *list_Z, 
		     int *list_NrmJ, int *list_rmJ[], 
		     int *maxZ, int *maxNlev, 
		     ElementP **Atomic_params);
void get_abund(char *abund_source, char *filemap_file, double **abund);
void fits_open(struct PARAMETERS *params, ElementP *Atomic_data, 
	       struct BIN_SPEC *spectrum);
void open_debug_output(struct DEBUG_FILES *debug);
void read_ion_bal(char *ionbal_file, struct DEBUG_FILES *debug, 
		  struct ION_BAL_TABLE *ionbal);
void init_rate_matrices(int maxNlev);
void init_trans_rates(int maxsize, struct TRANS_RATES *trans_rates);

void initialize(struct PARAMETERS *params, struct BIN_SPEC *spectrum,
		ElementP **Atomic_data, double **abund, 
		struct ION_BAL_TABLE *ionbal, 
		struct TRANS_RATES *trans_rates, 
		double **Lev_pop, 
		struct DEBUG_FILES *debug, int *maxZ, int *maxNlev) {

  /* Set up floating point exceptions */
#ifdef sparc
  /* Sun-type floating point exceptions */
  fp_except mask = FP_X_INV | FP_X_OFL | FP_X_DZ; 
  fp_except oldmask = fpsetmask(mask); 
#endif

  /* Open all debugging output files; does nothing if debug=NULL  */
  open_debug_output(debug);

  /* Create spectral bins in structure "spectrum" */
  create_spectral_bins(params->BinType, params->Nbin, 
		       params->BinMin, params->BinMax,
		       params->N_Elements, params->list_Z,
		       spectrum);
  
  /* Read atomic data into structure state */
  if (maxNlev != NULL) *maxNlev = 0;
  get_atomic_data(params->filemap_file, params->require_checksum, 
		  params->N_Elements, params->list_Z, 
		  params->list_NrmJ, params->list_rmJ, 
		  maxZ, maxNlev, Atomic_data);

  /* Read atomic abundances */
  get_abund(params->abund_source, params->filemap_file, abund);

  /* Read ionization balance, if needed. */
  if (params->use_ionbal_table) {
    read_ion_bal(params->ionbal_file, debug, ionbal);
  }

  /* Initialize FITS output files.  This is OPTIONAL; */
  /* Nothing in the library requires that these files are open. */
  
  fits_open(params, *Atomic_data, spectrum); 

  /* Malloc space for rate matricies used in solve_rate_eqns.c */
  init_rate_matrices(*maxNlev); 

  /* Malloc space for rate matricies used in calc_trans_rates.c */
  trans_rates->maxNlev = 0;
  init_trans_rates(*maxNlev, trans_rates);

  /* And the level population arrays */
  *Lev_pop = (double *) malloc(*maxNlev*sizeof(double));

}
