#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <fitsio.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

char *roman_element(int Z, int J);

/* Given an open fits file pointer, an atomic number Z, an ionization state */
/* J, and a datatype, move to the matching HDU. */

int moveto_ion_hdu(fitsfile *fptr, int Z,int rmJ,int datatype, char *ion_name){

  int done, found, typematch, skipped;
  int status=0;
  int hdunum;
  int maxhdu;
  int hdutype;
  int int_val;
  char str_val[MAXSTRLEN];

  skipped=FALSE;
  hdunum = 1;
  if (fits_movabs_hdu(fptr, hdunum, &hdutype, &status)) {
    fitsmess("moveto_ion_hdu",status,"Error moving to beginning of file.");
    status = 0;
  }
    
  if (fits_get_num_hdus(fptr, &maxhdu, &status)) {
    fitsmess("moveto_ion_hdu",status,"Error getting number of HDUs.");
    status = 0;
  }

  found = FALSE;
  while (!found) {
    done = FALSE;

    if (fits_movrel_hdu(fptr, 1, &hdutype, &status)) {
      message("moveto_ion_hdu","Ion : %s",roman_element(Z,rmJ-1));
      fitsmess("moveto_ion_hdu",status,"Error moving to %d-th HDU",hdunum+1);
      hdunum=1;
    } else {
      hdunum++;
    }
    
    if (fits_read_key(fptr, TSTRING, "HDUCLASS", str_val, NULL, &status)) {
      fitsmess("moveto_ion_hdu",status,"Error reading HDUCLASS key");
      status = 0;
    }
    
    typematch = FALSE;
      if (strncmp(str_val,HDUCLASS_ATOMIC,strlen(HDUCLASS_ATOMIC))==0) {
      
      if (fits_read_key(fptr, TSTRING, "HDUCLAS1", str_val, NULL, &status)) {
	fitsmess("moveto_ion_hdu",status,"Error reading HDUCLAS1 key");
	status = 0;
      }

      switch (datatype) {
      case (DATATYPE_IONREC):
	if (strncmp(str_val,IONREC_FITS,strlen(IONREC_FITS))==0)
	  typematch = TRUE;
	break;
      case (DATATYPE_LEVEL):
	if (strncmp(str_val,LEVEL_FITS,strlen(LEVEL_FITS))==0)
	  typematch = TRUE;
	break;
      case (DATATYPE_RADIATION):
	if (strncmp(str_val,RADIATION_FITS,strlen(RADIATION_FITS))==0)
	  typematch = TRUE;
	break;
      case (DATATYPE_E_COLLISION):
	if (strncmp(str_val,COLLISION_FITS,strlen(COLLISION_FITS))==0)
	  typematch = TRUE;
	break;
      case (DATATYPE_P_COLLISION):
	if (strncmp(str_val,COLLISION_FITS,strlen(COLLISION_FITS))==0)
	  typematch = TRUE;
	break;
      case (DATATYPE_SATELLITE):
	if (strncmp(str_val,SATELLITE_FITS,strlen(SATELLITE_FITS))==0)
	  typematch = TRUE;
	break;
      default:
	errmess("open_file_filemap","Illegal datatype %d",datatype);
	break;
      }
    }
      
    /* Ok, so at this point we know that the HDUCLASS and HDUCLAS1 types */
    /* are at least consistent.  That means the data we want to get could */
    /* be in this file.  Doesn't mean it has to be, necessarily, but it */
    /* could be. So we need to match Z and rmJ */ 
    
    if (typematch) {
      if (fits_read_key(fptr, TINT, FITSKEY_ELEMENT, &int_val,NULL, &status)) {
	fitsmess("moveto_ion_hdu",status,"Error reading %s key",
		 FITSKEY_ELEMENT);
	status=0;
      }
      
      if (Z==int_val) {
	if (fits_read_key(fptr, TINT,FITSKEY_IONSTAT,&int_val,NULL,&status)) {
	  status=0;
	  if (fits_read_key(fptr, TINT,FITSKEY_IONSTATE,&int_val,NULL,
			    &status)) {
	    fitsmess("moveto_ion_hdu",status,"Error reading %s key",
		     FITSKEY_IONSTATE);
	    status=0;
	  }
	}
      
	if (rmJ-1==int_val) { /* Bingo */
	  done = TRUE;
	  found = TRUE;
	  if (fits_read_key(fptr, TSTRING,FITSKEY_IONNAME, str_val,
			    NULL,&status)) {
	    fitsmess("moveto_ion_hdu",status,"Error reading %s key",
		     FITSKEY_IONNAME);
	    status=0;
	  } else {
	    strncpy(ion_name, str_val, 15);
	  }
	}
      }
    }
    if ((!found)&&(hdunum == maxhdu)) {  /* No match, end of file */
      found = TRUE; /* To get out of loop */
      skipped = TRUE; /* To signal nothing found */
    }
  }
  if (skipped) {
    found = FALSE; /* Didn't really find anything */
  }
  
  return found;
}
