
/* ******************************************************** */
/* print_table:
   function to print arrays of structure State*/
/* *******************************************************  */
#include <stdlib.h>
#include <stdio.h>

#include "apec.h"

void print_table(struct DEBUG_FILES *debug,
		 struct ELEMENT_ION_STATE *state) {

  int n, ni, nj, nk, n_st;
  long int nkl;

  int nstate,  nstate_max;
  int nionrec_max;
  int nnlev_max;
  int nnwv_max;
  int nexc_max;
  int nsatel_max;
  int nmax, i;
    
  if (debug == NULL) return;

  n_st = state[0].ion_inf->Z;
  if (debug->do_table && debug->ascii) {
    for (n=0; n < n_st; n++) {

      fprintf(debug->table.ascii, "%7s%4d%5d%5d%5d%6ld%6d%5d%5d\n",
	      state[n].ion_inf->Ion,
	      state[n].ion_inf->Z,
	      state[n].ion_inf->rmJ,
	      state[n].ion_inf->nionrec,
	      state[n].ion_inf->nlev,
	      state[n].ion_inf->nwv,
	      state[n].ion_inf->nexc,
	      state[n].ion_inf->npro,
	      state[n].ion_inf->nsatel);
      
      /* Loop through the lines of s?  in infile */
      
      for (ni=0; ni < state[n].ion_inf->nionrec; ni++) {
	fprintf(debug->table.ascii,"%8d%7d%7d%7d%7d%12.3e%12.3e%12.3e\n",
		state[n].ionrec_dat[ni].ion_final,
		state[n].ionrec_dat[ni].level_final,
		state[n].ionrec_dat[ni].level_init,
		state[n].ionrec_dat[ni].tr_type,
		state[n].ionrec_dat[ni].par_type,
		state[n].ionrec_dat[ni].min_temp,
		state[n].ionrec_dat[ni].max_temp,
		state[n].ionrec_dat[ni].IonRec_Par[0]);
      }
      
      /* Loop through the lines of level data */
      
      for (nj=0; nj < state[n].ion_inf->nlev; nj++) {
	fprintf(debug->table.ascii,"%4d%5d%7g%5d%5d%5e%5d\n",
		state[n].lev_dat[nj].lev,
		state[n].lev_dat[nj].deg,
		state[n].lev_dat[nj].energy,
		state[n].lev_dat[nj].N,
		state[n].lev_dat[nj].L,
		state[n].lev_dat[nj].S,
		state[n].lev_dat[nj].phottype);
      }
      
      /* Loop through the lines of radiative transition data */
      
      for (nkl=0; nkl < state[n].ion_inf->nwv; nkl++) {
	fprintf(debug->table.ascii,"%3d%3d%10g%10g\n",
		state[n].linerad_dat[nkl].lev_upper,
		state[n].linerad_dat[nkl].lev_lower,
		state[n].linerad_dat[nkl].lambda,
		state[n].linerad_dat[nkl].A);
      }
      
      /* Loop through the lines of collision strength data */
      
      for (nk=0; nk < state[n].ion_inf->nexc; nk++) {
	fprintf(debug->table.ascii,"%3d%3d%4d%3d%5d%10g%10g%10g%10g%10g\n",
		state[n].ion_inf->Z,
		state[n].ion_inf->rmJ,
		state[n].colltrns_dat[nk].lev_upper,
		state[n].colltrns_dat[nk].lev_lower,
		state[n].colltrns_dat[nk].coll_type,
		state[n].colltrns_dat[nk].om[0],
		state[n].colltrns_dat[nk].om[1],
		state[n].colltrns_dat[nk].om[2],
		state[n].colltrns_dat[nk].om[3],
		state[n].colltrns_dat[nk].om[4]);
      }
      
      /* Loop through the lines of the dielectronic satellite data */
      
      for (nk=0; nk < state[n].ion_inf->nsatel; nk++) {
	fprintf(debug->table.ascii,"%3d%3d%4d%4d%5d%10g%10g%10g\n",
		state[n].ion_inf->Z,
		state[n].ion_inf->rmJ,
		state[n].satellite_dat[nk].lev_upper,
		state[n].satellite_dat[nk].lev_lower,
		state[n].satellite_dat[nk].type,
		state[n].satellite_dat[nk].lambda,
		state[n].satellite_dat[nk].q_exc,
		state[n].satellite_dat[nk].E_excite);
      }
      
      fprintf(debug->table.ascii,"\n");
      
    }
    fflush(debug->table.ascii);
  }

  nstate_max  = state[0].ion_inf->Z;
  if (debug->do_table && debug->fits) {
    for (nstate=0; nstate < nstate_max; nstate++) {
  
      nionrec_max = state[nstate].ion_inf->nionrec;
      nnlev_max   = state[nstate].ion_inf->nlev;
      nnwv_max    = state[nstate].ion_inf->nwv;
      nexc_max    = state[nstate].ion_inf->nexc;
      nsatel_max  = state[nstate].ion_inf->nsatel;
      nmax = nionrec_max; if (nnlev_max > nmax) nmax = nnlev_max;
      if (nnwv_max > nmax) nmax = nnwv_max; if (nexc_max > nmax) nmax = nexc_max;
      if (nsatel_max > nmax) nmax = nsatel_max;

      i = 0;

      while (i < nmax) {
	fd_write_row(&debug->table.fits,
		     /* general info */
		     state[nstate].ion_inf->Ion,
		     state[nstate].ion_inf->Z,
		     state[nstate].ion_inf->rmJ,
		     state[nstate].ion_inf->nionrec,
		     state[nstate].ion_inf->nlev,
		     state[nstate].ion_inf->nwv,
		     state[nstate].ion_inf->nexc,
		     state[nstate].ion_inf->npro,
		     state[nstate].ion_inf->nsatel,
		     /* Loop through the lines of s?  in infile */
		     /*	fprintf(debug->table,"%8d%7d%7d%7d%7d%12.3e%12.3e%12.3e\n", */
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].ion_final : 0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].level_final : 0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].level_init : 0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].tr_type : 0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].par_type : 0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].min_temp : 0.0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].max_temp : 0.0,
		     (i < nionrec_max) ? state[nstate].ionrec_dat[i].IonRec_Par[0] : 0.0,
		     /* Loop through the lines of level data */
		     /* fprintf(debug->table,"%4d%5d%7g%5d%5d%5e%5d\n", */
		     (i < nnlev_max) ? state[nstate].lev_dat[i].lev : 0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].deg : 0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].energy : 0.0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].N : 0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].L : 0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].S : 0.0,
		     (i < nnlev_max) ? state[nstate].lev_dat[i].phottype : 0,
		     /* Loop through the lines of radiative transition data */
		     /* fprintf(debug->table,"%3d%3d%10g%10g\n", */
		     (i < nnwv_max) ? state[nstate].linerad_dat[i].lev_upper : 0,
		     (i < nnwv_max) ? state[nstate].linerad_dat[i].lev_lower : 0,
		     (i < nnwv_max) ? state[nstate].linerad_dat[i].lambda : 0.0,
		     (i < nnwv_max) ? state[nstate].linerad_dat[i].A : 0.0,
		     /* Loop through the lines of collision strength data */
		     /* fprintf(debug->table,"%3d%3d%4d%3d%5d%10g%10g%10g%10g%10g\n", */
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].lev_upper : 0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].lev_lower : 0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].coll_type : 0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].om[0] : 0.0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].om[1] : 0.0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].om[2] : 0.0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].om[3] : 0.0,
		     (i < nexc_max) ? state[nstate].colltrns_dat[i].om[4] : 0.0,
		     /* Loop through the lines of the dielectronic satellite data */
		     /* fprintf(debug->table,"%3d%3d%4d%4d%5d%10g%10g%10g\n", */
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].lev_upper : 0,
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].lev_lower : 0,
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].type : 0,
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].lambda : 0.0,
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].q_exc : 0.0,
		     (i < nsatel_max) ? state[nstate].satellite_dat[i].E_excite : 0.0);
	i++;
      }
    }
  }

}
