FUNCTION select_blend, data, lambda, resolution, temperature, $
                       ELEMENT=element, ION=ion, SILENT=silent

ERGperKEV = 1.60219e-9

if (n_params() lt 4) then begin
    print,'Call as'
    print,'result = select_blend(data, wavelength, resolution, temperature, ELEMENT=element, ION=ion, /SILENT)'
    print,'The result will be emissivity with the given ionic state and the'
    print,'wavelengths within the resolution size.   Set /SILENT if you do'
    print,'not want any error messages.'
    print,'If resolution > 10, it is assumed to mean lambda/dlambda.  If '
    print,'less than 10, it is taken to be the dlambda width to search.'
    return,' '
endif

if (n_elements(data) eq 0) then begin
    print,'No data in the data variable.'
    result = -1
    return, result
endif

if (n_tags(data) eq 0) then begin
    print,'Data is not a structure.'
    result = -1
    return, result
endif

if (n_elements(element) ne 0) then begin
    gp = where(data.element eq element,count)
    if (count eq 0) then begin
        if (n_elements(silent) ne 0) then $
          print,'No data in structure for Z=',element
        return, -1
    endif else begin
        data = data(gp)
    endelse
endif
if (n_elements(ion) ne 0) then begin
    gp = where(data.ion eq ion,count)
    if (count eq 0) then begin
        if (n_elements(silent) ne 0) then $
          print,'No data in structure for rmJ=',ion
        return, -1
    endif else begin
        data = data(gp)
    endelse
endif
;
; Now, find the temperature list
; 
T = data.temperature
T = T(uniq(T(sort(T))))
temperature = T
;
;  Now, work out resolution element
;
if (resolution gt 10) then begin
    dlambda = lambda/resolution
endif else begin
    dlambda = resolution
endelse

result = 0*temperature
for iT = 0,n_elements(temperature)-1 do begin
    gp = where(abs(data.temperature - temperature(iT)) le 1,count)
    if (count eq 0) then begin
        result(iT) = 0.0 
    endif else begin
        dataT = data(gp)
        gp = where(abs(dataT.lambda - lambda) le dlambda,count)
        if (count eq 0) then begin
            result(iT) = 0.0
        endif else begin
            result(iT) = total(dataT(gp).epsilon)
        endelse
    endelse
endfor

return, result
end
