PRO zscale_ecoll, Z, rmJ, outfile, ZARR=Zarr
;
;  First, get values to scale from
;
if (n_elements(Zarr) eq 0) then Zarr = [Z-1,Z+1]
rmJarr = Zarr - (Z - rmJ)
;
;  Now, load these files.
;
read_ecoll, Zarr(0), rmJarr(0), base_data, base_header, FOUND=found,/silent
if (found eq 0) then begin
    print,strcompress('Cannot find required ion '+$
                      roman_element(Zarr(0),rmJarr(0)))
    return
endif
;
;  Assume the Zarr(0) ion is a good estimate for everything
;  except the temperature and collision strengths.  
;
Ntr = n_elements(base_data)
scale_data = base_data
N_T = 20

for iTr=0,Ntr-1 do scale_data(iTr).Coeff_Type = 100 + N_T 
for iTr=0,Ntr-1 do scale_data(iTr).Temperature = fltarr(N_T)
for iTr=0,Ntr-1 do scale_data(iTr).Coeff_OM = fltarr(N_T)
for iTr=0,Ntr-1 do scale_data(iTr).Reference = 'APED Approximation'
;
;  Now, fit the level energy values from the Zarr, rmJarr ions
;
Uvec = fltarr(n_elements(Zarr),N_T)

for iTr=0,Ntr-1 do begin
    ll = scale_data(iTr).lower_lev
    lu = scale_data(iTr).upper_lev
    T =  make_range(scale_data(iTr).Min_Temp,scale_data(iTr).Max_Temp,N_T,/log)

    for iZ=0,n_elements(Zarr)-1 do Uvec(iZ,*)=$
      calc_upsilon(Zarr(iZ),rmJarr(iZ),ll,lu,T,/silent)*(Zarr(iZ)^2)
    for iT=0,N_T-1 do begin
        c = poly_fit(Zarr, reform(Uvec(*,iT)), 1)
        scale_data(iTr).Temperature(iT) = T(iT)
        scale_data(iTr).Coeff_OM(iT)    = poly(Z, c)/(Z^2)
    endfor
endfor
;
;  Write out results
;
ions_used = ' '
for iZ=0,n_elements(Zarr)-1 do $
  ions_used = ions_used+' '+roman_element(Zarr(iZ),rmJarr(iZ))
ions_used = strtrim(strcompress(ions_used),2)

comments = ['COMMENT    Approximated via fitting collision strength data ',$
            'COMMENT    from '+ions_used,$
            'COMMENT    Using program zscale_ecoll.pro']

write_ecoll, outfile, Z, rmJ, scale_data, COMMENTS=comments
return
end
