PRO zscale_elev, Z, rmJ, outfile, ZARR=Zarr
;
;  First, get values to scale from
;
if (n_elements(Zarr) eq 0) then Zarr = [Z-1,Z+1]
rmJarr = Zarr - (Z - rmJ)
;
;  Now, load these files.
;
read_elev, Z-1, rmJ-1, dataM1, headerM1, FOUND=found,/silent
if (found eq 0) then begin
    print,strcompress('Cannot find required ion '+roman_element(Z-1,rmJ-1))
    return
endif
;
;  Assume the ion (minus 1 proton) is a good estimate for everything
;  except the energy and photoionization rates.
;
Nlev = n_elements(dataM1)
scale_data = dataM1
for iLev=0,Nlev-1 do scale_data.energy = 0.0
scale_data.phot_type = -1  ; No data available
for iLev=0,Nlev-1 do scale_data(iLev).phot_par = fltarr(20)
for iLev=0,Nlev-1 do scale_data(iLev).reference = 'APED Approximation'
;
;  Now, fit the level energy values from the Zarr, rmJarr ions
;
Evec = fltarr(n_elements(Zarr))
for iLev=0,Nlev-1 do begin
    for iZ=0,n_elements(Zarr)-1 do Evec(iZ)=$
      calc_energy(Zarr(iZ),rmJarr(iZ),iLev+1,/silent)/(Zarr(iZ)*Zarr(iZ))
    c = poly_fit(Zarr, Evec, 1)
    scale_data(iLev).energy = poly(Z, c)*Z*Z
endfor
;
;  Write out results
;
ions_used = ' '
for iZ=0,n_elements(Zarr)-1 do $
  ions_used = ions_used+' '+roman_element(Zarr(iZ),rmJarr(iZ))
ions_used = strtrim(strcompress(ions_used),2)

comments = ['COMMENT    Approximated via fitting energy level data from',$
            'COMMENT    '+ions_used,$
            'COMMENT    Using program zscale_elev.pro']

write_elev, outfile, Z, rmJ, scale_data, COMMENTS=comments
return
end
