%
%
%
%
% R. Hoogerwerf, SAO, 04/26/2002
%

uses("MATH-arrays");
use_namespace("Math");


%----
variable indx_round = 0;
variable indx_floor = 1;
variable indx_ceil = 2;

define indx() {

   variable x, arr, which = Math->indx_round;

   %
   % Chech arguments
   %
   switch (_NARGS) 
   { case 2 : (x, arr) = (); }
   { case 3 : (x, arr, which) = (); }
   { message("Usage: Math->indx(x, array [, which]);");
     _pop_n(_NARGS);
     return -1; }

   %
   % Check types
   %
   if (typeof(x)   == Array_Type and
       typeof(arr) != Array_Type) {
      variable tmp = x;
      x = arr;
      arr = tmp;
   } else if (typeof(arr) != Array_Type) {
      message("Math->indx: did not receive an array as an argument.");
      return -2;
   }

   variable lower = 0, upper = length(arr)-1, middle;

   if (x < arr[lower]) return -3;
   if (x > arr[upper]) return -4;

   while (upper - lower > 1) {
      middle = (upper + lower) / 2;
      if (x > arr[middle]) {
	 lower = middle;
      } else {
	 upper = middle;
      }
   }

   switch (which) 
   { case indx_round : 
	 if ((x - arr[lower]) < (arr[upper] - x)) {
	    return lower;
	 } else {
	    return upper;
	 }}
   { case indx_floor : return lower; }
   { case indx_ceil  : return upper; }
   { message("Math->indx: incorrect which argument"); return -5; }

}

%----

define _min(x, y) {

   if (x <= y) {
      return x;
   } else {
      return y;
   }
}

%----

define _max(x, y) {

   if (x >= y) {
      return x;
   } else {
      return y;
   }
}

%----

define min() {

   %
   % Returns the minimum of the arguments
   %
   variable i, arg, min;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      arg = ();
      if (Array_Type == typeof(arg)) {
	 min = arg[0];
	 for (i = 1; i < length(arg); i++) min = Math->_min(min,arg[i]);
	 return min;
      } else {
	 return arg;
      }
   } else {
      min = ();
      for (i = 2; i <= _NARGS; i++) {
	 arg = ();
	 min = Math->_min(min,arg);
      }
      return min;
   }
}

%----

define max() {

   %
   % Returns the maximum of the arguments
   %
   variable i, arg, max;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      arg = ();
      if (Array_Type == typeof(arg)) {
	 max = arg[0];
	 for (i = 1; i < length(arg); i++) max = Math->_max(max,arg[i]);
	 return max;
      } else {
	 return arg;
      }
   } else {
      max = ();
      for (i = 2; i <= _NARGS; i++) {
	 arg = ();
	 max = Math->_max(max,arg);
      }
      return max;
   }
}

%----

define sum() {

   %
   % Returns the sum of the arguments.
   %
   variable i, arg, sum;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      arg = ();
      if (Array_Type == typeof(arg)) {
	 sum = 0.0;
	 for (i = 0; i < length(arg); i++) sum += arg[i];
	 return sum;
      } else {
	 return arg;
      }
   } else {
      sum = 0.0;
      for (i = 1; i <= _NARGS; i++) {
	 arg = ();
	 sum += arg;
      }
      return sum;
   }
}

%----

define avg() {

   %
   % Returns the avg of the arguments.
   %
   variable i, arg, avg;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      arg = ();
      if (Array_Type == typeof(arg)) {
	 avg = 0.0;
	 for (i = 0; i < length(arg); i++) avg += arg[i];
	 return avg/typecast(length(arg),Double_Type);
      } else {
	 return arg;
      }
   } else {
      avg = 0.0;
      for (i = 1; i <= _NARGS; i++) {
	 arg = ();
	 avg += arg;
      }
      return avg/typecast(_NARGS,Double_Type);
   }
}

%----

define avg2() {

   %
   % Returns the avg2 of the arguments.
   %
   variable i, arg, avg2;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      arg = ();
      if (Array_Type == typeof(arg)) {
	 avg2 = 0.0;
	 for (i = 0; i < length(arg); i++) avg2 += arg[i]^2;
	 return avg2/typecast(length(arg),Double_Type);
      } else {
	 return arg^2;
      }
   } else {
      avg2 = 0.0;
      for (i = 1; i <= _NARGS; i++) {
	 arg = ();
	 avg2 += arg^2;
      }
      return avg2/typecast(_NARGS,Double_Type);
   }
}

%----

define variance() {

   variable args = __pop_args(_NARGS);
   
   print(args);

   variable  x2 = avg2(__push_args(args));
   variable _x2 = avg(__push_args(args));
   _x2 = _x2 * _x2;

   print(x2);
   print(_x2);

   return x2 - _x2;   

}

%----

define median() {

   %
   % Returns the median of the arguments.
   %
   variable i, arg, median;

   if (_NARGS == 0) {
      return 0;
   } else if (_NARGS == 1) {
      %
      median = ();
      if (Array_Type != typeof(median)) {
	 return median;
      }
   } else {
      median = Double_Type[_NARGS];
      for (i = 0; i < _NARGS; i++) {
	 arg = ();
	 median[i] = arg;
      }
   }

   % Sort array in ascending order
   variable I = array_sort(median,&ascend);
   median = median[I];

   variable n = length(median);
   if (n == 1) {
      median = median[0];
   } else if (is_odd(n)) {
      % this is the easy case
      median = median[(n-1)/2];
   } else {
      % this is slightly harder
      median = (median[n/2] + median[n/2 - 1]) / 2.0;
   }

   return median;

}

%----

private define ascend (x, y) {

   return x - y;
      
}
 
%----

private define descend (x, y) {

   return y - x;
      
}
 
%----
%----
%----
%----
%----
%----
%----
%----
%----
%----
%----
%----
%----
