#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>

#include "apec.h"
#include "_apec_slang.h"



/*



*/


float power_law(float gamma,
		float ampl,
		float x) {
  
  return(ampl*pow(x,-gamma));
 
}

float broken_power_law(float gamma1,
		       float gamma2,
		       float xbreak,
		       float ampl,
		       float x) {

  if (x <= xbreak) {
    return(ampl*pow(x,-gamma1));
  } else {
    return(ampl*pow(xbreak,gamma2-gamma1)*pow(x,-gamma2));
  }

}

float double_power_law(float gamma,
		       float beta,
		       float alpha,
		       float xbreak,
		       float ampl,
		       float x) {

  float res;

  res = ampl * 
    pow(x/xbreak,-gamma) * 
    pow(1+pow(x/xbreak,1.0/alpha),-(beta-gamma)*alpha);

  return(res);

}


int _em_power_law_(char *name,
		   float gamma,
		   float ampl,
		   int nT,
		   float Tmin,
		   float Tmax,
		   int log,
		   float density,
		   float *abundances,
		   float redshift,
		   int flag,
		   int keV,
		   long nlinelist,
		   long *linelist,
		   int nbin,
		   float *binlo, 
		   float *binhi, 
		   float *spectrum) {
  

  int i, failed=0;
  float *Tarr, dT;
  float *normarr, *densarr;


  /* first name array of temperatures */

  if (Tmin <= 0.0 || Tmax <= 0.0) {
    return(-1);
  }

  Tarr = (float *)malloc(sizeof(float)*nT);
  if (nT == 1) {
    Tarr[0] = Tmin;
  } else {
    if (log == TRUE) {
      dT = (log10(Tmax)-log10(Tmin))/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = pow(10.0,log10(Tmin) + dT*(float)i);
    } else {
      dT = (Tmax-Tmin)/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = Tmin + dT*(float)i;
    }
  }

    

  /* Evaluate powerlaw EM to assign normalizations of em model */
  normarr = (float *)malloc(sizeof(float)*nT);
  densarr = (float *)malloc(sizeof(float)*nT);
  for (i=0;i<nT;i++) {
    normarr[i] = power_law(gamma,ampl,Tarr[i]);
    densarr[i] = density;
    /* printf("%d %f %f\n", i, Tarr[i], normarr[i]); */
  }

  /* for (i=0;i<nT;i++) printf("%d %f %f\n",i,Tarr[i],normarr[i]); */

  if (-1 == extract_ncomp_spectrum_for_modelname(name,nT,Tarr,densarr,normarr,abundances,redshift,flag,keV,nlinelist,linelist,nbin,binlo,binhi,spectrum)) {
    failed = 1;
    goto finish;
  }

 finish:
  /* free allocated memory */
  free(Tarr);
  free(normarr);
  free(densarr);

  if (failed)
    return(-1);
  else
    return (0);

}



int _em_broken_power_law_(char *name,
			  float gamma1,
			  float gamma2,
			  float Tbreak,
			  float ampl,
			  int nT,
			  float Tmin,
			  float Tmax,
			  int log,
			  float density,
			  float *abundances,
			  float redshift,
			  int flag,
			  int keV,
			  long nlinelist,
			  long *linelist,
			  int nbin,
			  float *binlo, 
			  float *binhi, 
			  float *spectrum) {
  

  int i, failed=0;
  float *Tarr, dT;
  float *normarr, *densarr;


  /* first name array of temperatures */

  if (Tmin <= 0.0 || Tmax <= 0.0) {
    return(-1);
  }

  Tarr = (float *)malloc(sizeof(float)*nT);
  if (nT == 1) {
    Tarr[0] = Tmin;
  } else {
    if (log == TRUE) {
      dT = (log10(Tmax)-log10(Tmin))/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = pow(10.0,log10(Tmin) + dT*(float)i);
    } else {
      dT = (Tmax-Tmin)/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = Tmin + dT*(float)i;
    }
  }

    

  /* Evaluate powerlaw EM to assign normalizations of em model */
  normarr = (float *)malloc(sizeof(float)*nT);
  densarr = (float *)malloc(sizeof(float)*nT);
  for (i=0;i<nT;i++) {
    normarr[i] = broken_power_law(gamma1,gamma2,Tbreak,ampl,Tarr[i]);
    densarr[i] = density;
  }

  /* for (i=0;i<nT;i++) printf("%d %f %f\n",i,Tarr[i],normarr[i]); */

  if (-1 == extract_ncomp_spectrum_for_modelname(name,nT,Tarr,densarr,normarr,abundances,redshift,flag,keV,nlinelist,linelist,nbin,binlo,binhi,spectrum)) {
    failed = 1;
    goto finish;
  }

 finish:
  /* free allocated memory */
  free(Tarr);
  free(normarr);
  free(densarr);

  if (failed)
    return(-1);
  else
    return (0);

}




int _em_double_power_law_(char *name,
			  float gamma,
			  float beta,
			  float alpha,
			  float Tbreak,
			  float ampl,
			  int nT,
			  float Tmin,
			  float Tmax,
			  int log,
			  float density,
			  float *abundances,
			  float redshift,
			  int flag,
			  int keV,
			  long nlinelist,
			  long *linelist,
			  int nbin,
			  float *binlo, 
			  float *binhi, 
			  float *spectrum) {


  int i, failed=0;
  float *Tarr, dT;
  float *normarr, *densarr;


  /* first name array of temperatures */

  if (Tmin <= 0.0 || Tmax <= 0.0) {
    return(-1);
  }

  Tarr = (float *)malloc(sizeof(float)*nT);
  if (nT == 1) {
    Tarr[0] = Tmin;
  } else {
    if (log == TRUE) {
      dT = (log10(Tmax)-log10(Tmin))/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = pow(10.0,log10(Tmin) + dT*(float)i);
    } else {
      dT = (Tmax-Tmin)/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = Tmin + dT*(float)i;
    }
  }

    

  /* Evaluate powerlaw EM to assign normalizations of em model */
  normarr = (float *)malloc(sizeof(float)*nT);
  densarr = (float *)malloc(sizeof(float)*nT);
  for (i=0;i<nT;i++) {
    normarr[i] = double_power_law(gamma,beta,alpha,Tbreak,ampl,Tarr[i]);
    densarr[i] = density;
  }

  /* for (i=0;i<nT;i++) printf("%d %f %f\n",i,Tarr[i],normarr[i]); */

  if (-1 == extract_ncomp_spectrum_for_modelname(name,nT,Tarr,densarr,normarr,abundances,redshift,flag,keV,nlinelist,linelist,nbin,binlo,binhi,spectrum)) {
    failed = 1;
    goto finish;
  }

 finish:
  /* free allocated memory */
  free(Tarr);
  free(normarr);
  free(densarr);

  if (failed)
    return(-1);
  else
    return (0);

}


float chebyshev(int n, float x) {

  float res;

  switch(n) {
     case  0 : res = 1.0;
     case  1 : res = x;
     case  2 : res = -1.0+2.0*pow(x,2);
     case  3 : res = -3.0*x + 4.0*pow(x,3);
     case  4 : res = 1.0 - 8.0*pow(x,3) + 8.0*pow(x,4);
     case  5 : res = 5.0*x - 20.0*pow(x,3) + 16.0*pow(x,5);
     case  6 : res = -1.0 + 18.0*pow(x,3) - 48.0*pow(x,4) + 32.0*pow(x,6);
     case  7 : res = -7.0*x + 56.0*pow(x,3) -112.0*pow(x,5) + 64.0*pow(x,7);
     case  8 : res = 1.0 - 32.0*pow(x,3) + 160.0*pow(x,4) - 256.0*pow(x,6) + 128.0*pow(x,8);
     case  9 : res = 9.0*x - 120.0*pow(x,3) + 432.0*pow(x,5) - 576.0*pow(x,7) + 256*pow(x,9);
     case 10 : res = -1.0 + 50*pow(x,3) - 400.0*pow(x,4) + 1120.0*pow(x,6) - 1280.0*pow(x,8) + 512.0*pow(x,10);
     default : res = 2.0*x*chebyshev(n-1,x) - chebyshev(n-2,x);
  }

  return(res);

}

float polynomial(int n, float x) {

  float res;

  switch(n) {
     case  0 : res = 1.0;
     case  1 : res = x;
     default : res = pow(x,n);
  }

  return(res);

}

int _em_chebyshev_(char *name,
		   int n,
		   float *ampl,       /* array of length n containing  
					 amplitudes of chebyshev polynomials */
		   int nT,
		   float Tmin,
		   float Tmax,
		   int log,
		   float density,
		   float *abundances,
		   float redshift,
		   int flag,
		   int keV,
		   long nlinelist,
		   long *linelist,
		   int nbin,
		   float *binlo, 
		   float *binhi, 
		   float *spectrum) {
  

  int i, j, failed=0;
  float *Tarr, dT;
  float *xarr;
  float *normarr, *densarr;


  /* first name array of temperatures */

  if (Tmin <= 0.0 || Tmax <= 0.0) {
    return(-1);
  }

  Tarr = (float *)malloc(sizeof(float)*nT);
  xarr = (float *)malloc(sizeof(float)*nT);
  if (nT == 1) {
    Tarr[0] = Tmin;
  } else {
    if (log == TRUE) {
      dT = (log10(Tmax)-log10(Tmin))/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = pow(10.0,log10(Tmin) + dT*(float)i);
    } else {
      dT = (Tmax-Tmin)/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = Tmin + dT*(float)i;
    }
  }

  /* Chebyshev polynomials are defined for x \in [-1,1]
     so we need to scale Tmin and Tmax to -1 and 1 */
  for (i=0;i<nT;i++) xarr[i] = -1.0 + 2.0*(Tarr[i] - Tmin)/(Tmax-Tmin);

  /* Evaluate powerlaw EM to assign normalizations of em model */
  normarr = (float *)malloc(sizeof(float)*nT);
  densarr = (float *)malloc(sizeof(float)*nT);
  for (i=0;i<nT;i++) { 
    normarr[i] = 0.0;
    for (j=0;j<n;j++) normarr[i] += ampl[j]*chebyshev(j,xarr[i]);
    densarr[i] = density;
  }

  /* Check we cannot deal with negative normalizations, if this happens
     spit out a warning and set the normalization to 0 */
  for (i=0;i<nT;i++) { 
    if (normarr[i] < 0.0) {
      normarr[i] = 0.0;
      printf("Coefficient %d of the Chebyshev EM is negative\nSetting c%d = 0.0",i,i);
    }
  }



  /* for (i=0;i<nT;i++) printf("%d %f %f\n",i,Tarr[i],normarr[i]); */

  if (-1 == extract_ncomp_spectrum_for_modelname(name,nT,Tarr,densarr,normarr,abundances,redshift,flag,keV,nlinelist,linelist,nbin,binlo,binhi,spectrum)) {
    failed = 1;
    goto finish;
  }

 finish:
  /* free allocated memory */
  free(Tarr);
  free(xarr);
  free(normarr);
  free(densarr);

  if (failed)
    return(-1);
  else
    return (0);

}


int _em_polynomial_(char *name,
		    int n,
		    float *ampl,       /* array of length n containing  
					  amplitudes of chebyshev polynomials */
		    int nT,
		    float Tmin,
		    float Tmax,
		    int log,
		    float density,
		    float *abundances,
		    float redshift,
		    int flag,
		    int keV,
		    long nlinelist,
		    long *linelist,
		    int nbin,
		    float *binlo, 
		    float *binhi, 
		    float *spectrum) {
  

  int i, j, failed=0;
  float *Tarr, dT;
  float *normarr, *densarr;


  /* first name array of temperatures */

  if (Tmin <= 0.0 || Tmax <= 0.0) {
    return(-1);
  }

  Tarr = (float *)malloc(sizeof(float)*nT);
  if (nT == 1) {
    Tarr[0] = Tmin;
  } else {
    if (log == TRUE) {
      dT = (log10(Tmax)-log10(Tmin))/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = pow(10.0,log10(Tmin) + dT*(float)i);
    } else {
      dT = (Tmax-Tmin)/(float)(nT-1);
      for (i=0;i<nT;i++) Tarr[i] = Tmin + dT*(float)i;
    }
  }


  /* Evaluate powerlaw EM to assign normalizations of em model */
  normarr = (float *)malloc(sizeof(float)*nT);
  densarr = (float *)malloc(sizeof(float)*nT);
  for (i=0;i<nT;i++) { 
    normarr[i] = 0.0;
    for (j=0;j<n;j++) normarr[i] += ampl[j]*polynomial(j,Tarr[i]);
    densarr[i] = density;
  }

  /* Check we cannot deal with negative normalizations, if this happens
     spit out a warning and set the normalization to 0 */
  for (i=0;i<nT;i++) { 
    if (normarr[i] < 0.0) {
      normarr[i] = 0.0;
      printf("Coefficient %d of the Polynomial EM is negative\nSetting c%d = 0.0",i,i);
    }
  }

  /* for (i=0;i<nT;i++) printf("%d %f %f\n",i,Tarr[i],normarr[i]); */

  if (-1 == extract_ncomp_spectrum_for_modelname(name,nT,Tarr,densarr,normarr,abundances,redshift,flag,keV,nlinelist,linelist,nbin,binlo,binhi,spectrum)) {
    failed = 1;
    goto finish;
  }

 finish:
  /* free allocated memory */
  free(Tarr);
  free(normarr);
  free(densarr);

  if (failed)
    return(-1);
  else
    return (0);

}
