#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>
#include <fitsio.h>

#include "apec.h"
#include "_apec_slang.h"


/* 
   global variables holding all apec models and some book keeping
   counters;
*/
char *strtrim(char *string) {

  int n = strlen(string), i, j = 0;
  int first = 0;

  /* first get rid if bracketing qoutes ' */
  for (i=0; i < n; i++) {
    string[j] = string[i];
    if (string[i] == '\'' && first) {
      string[j] = '\0';
      j++;
    }
    if (string[i] == '\'' && !first) first = 1;
    if (!(string[i] == '\'')) j++;
    
  }

  /* second get rid of trailing spaces */
  n = strlen(string);

  first = 0;
  for (i=n-1; i>=0; i--) {
    if (string[i] == ' ' && !first) {
      string[i] = '\0';
    } else {
      first = 1;
    }
  }


  return(string);

}

int decode_element(char *string) {
  /* we are doing this brute force. There might be better ways
     to do this, but this is definitely easiest                 
  */
  char *Element[30] = {"H","He","Li","Be","B","C","N","O","F","Ne",
		       "Na","Mg","Al","Si","P","S","Cl","Ar","K","Ca",
		       "Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"};
  int i, elem=0;

  for (i=0; i<30; i++) 
    if (strncmp(string,Element[i],strlen(Element[i])) == 0) elem = i+1;

  return(elem);

}

int decode_ion(char *string) {
  /* we are doing this brute force. There might be better ways
     to do this, but this is definitely easiest                 
  */
  char *Element[30] = {"H","He","Li","Be","B","C","N","O","F","Ne",
		       "Na","Mg","Al","Si","P","S","Cl","Ar","K","Ca",
		       "Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"};
  char *Numeral[30] = {"I","II","III","IV","V","VI","VII","VIII","IX","X",
		       "XI","XII","XIII","XIV","XV","XVI","XVII","XVIII",
		       "XIX","XX","XXI","XXII","XXIII","XXIV","XXV","XXVI",
		       "XXVII","XXVIII","XXIX","XXX"};
  char name[20];
  int i, j, ion=0;

  for (i=0; i<30; i++) {
    if (strncmp(string,Element[i],strlen(Element[i])) == 0) {
      /* found correct element */
      for (j = 0; j < i+1; j++) {
	sprintf(name,"%s %s",Element[i],Numeral[j]);
	if (strncmp(string,name,strlen(name)) == 0) ion = j+1;
      }
    }
  }
  return(ion);

}

int get_ionpop_indx(int Z,int rmnJ) {

  int i, spot=0;

  for (i=1;i<Z;i++) spot += i + 1;
  for (i=1;i<rmnJ;i++) spot++;

  return(spot);

}

Apec_Model *setup_kT_density_grid(Apec_Model *am) {

  int i, j, k, l;
  int tmin, tmax, dmin, dmax, newT, newD;
 
  /* First allocate memory */
  am->grid_kT = (float *)malloc(sizeof(float)*am->num_temp);
  am->grid_density = (float *)malloc(sizeof(float)*am->num_dens);
  am->grid_kT_density = (int **)malloc(sizeof(int *)*am->num_temp);
  for (i= 0; i<am->num_temp; i++) {
    am->grid_kT_density[i] = (int *)malloc(sizeof(int)*am->num_dens);
  }

  /* we cannot assume that either the density of temperature array,
     as calculated and output by APEC is ordered in ascending or 
     descending order. So we need to do this ourselfs. If we are dealing
     with a NEI calculation things will be different; don't know yet
     how to deal with that.
  */
  
  /* since we assume no order in the temps and densities we'll do this
     brute force. We only need to to this once and it will probably not
     take up that much time
  */

  tmin = 0;
  tmax = 1;
  dmin = 0;
  dmax = 1;

  for (i=0;i<am->num_temp;i++) 
    for (j=0;j<am->num_dens;j++) 
      am->grid_kT_density[i][j] = -1;
    
  /* fill first element in array so we can start the filling process */
  am->grid_kT[0] = am->kT[0];
  am->grid_density[0] = am->density[0];
  am->grid_kT_density[0][0] = 0;

  for (k=0; k<am->n; k++) {
    /* first check it we are dealing with a new temperature? */
    newT = -1;
    for(i=tmin; i<tmax; i++) if (am->grid_kT[i] == am->kT[k]) newT = i;

    if (newT == -1) {
      /* insert new temperature row */
      i = 0;
      while (am->grid_kT[i] < am->kT[k] && i < tmax) i++;
      newT = i;
      if (newT != tmax) {
	/* shift rows to the left of i */
	for (j=am->num_temp-1;j>newT;j--) {
	  am->grid_kT[j] = am->grid_kT[j-1];
	  for (l=0;l<am->num_dens;l++) 
	    am->grid_kT_density[j][l] = am->grid_kT_density[j-1][l];
	}
      }
      am->grid_kT[newT] = am->kT[k];
      tmax++;
    } 

    /* Now take care of density component */
    newD = -1;
    for(i=dmin; i<dmax; i++) if (am->grid_density[i] == am->density[k]) newD = i;

    if (newD == -1) {
      /* insert new density column */
      i = 0;
      while (am->grid_density[i] < am->density[k] && i < dmax) i++;
      newD = i;
      if (newD != dmax) {
	/* shift columns down of i */
	for (j=am->num_dens-1;j>newD;j--) {
	  am->grid_density[j] = am->grid_density[j-1];
	  for (l=0;l<am->num_temp;l++) 
	    am->grid_kT_density[l][j] = am->grid_kT_density[l][j-1];
	}
      }
      am->grid_density[newD] = am->density[k];
      dmax++;
    } 

    am->grid_kT_density[newT][newD] = k;

  }

  return(am);

}


int read_line_info(fitsfile *fp, Line_Type *line,long n) {


  int status=0, colnum=0, anynul=0;
  char *nularray;

  /* first allocate the correct amount of memory */

  line->n = n;
  line->lambda = (float *)malloc(sizeof(float)*n);
  line->lambda_err = (float *)malloc(sizeof(float)*n);
  line->epsilon = (float *)malloc(sizeof(float)*n);
  line->epsilon_err = (float *)malloc(sizeof(float)*n);
  line->element = (long *)malloc(sizeof(long)*n);
  line->ion = (long *)malloc(sizeof(long)*n);
  line->upperlev = (long *)malloc(sizeof(long)*n);
  line->lowerlev = (long *)malloc(sizeof(long)*n);
  nularray = (char *)malloc(sizeof(char)*n);

  /* Now get column positions and read their content */
  if (fits_get_colnum(fp, CASEINSEN, "Lambda", &colnum, &status)) {
    printf("Could not determine column number for (Lambda).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_flt(fp, colnum, 1l, 1l, n, 0.0, line->lambda, &anynul, &status)) { 
    printf("Failed to read column %d (Lambda).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Lambda_Err", &colnum, &status)) {
    printf("Could not determine column number for (Lambda_Err).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_colnull_flt(fp, colnum, 1l, 1l, n, line->lambda_err, nularray, &anynul, &status)) { 
    printf("Failed to read column %d (Lambda_Err).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Epsilon", &colnum, &status)) {
    printf("Could not determine column number for (Epsilon).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_flt(fp, colnum, 1l, 1l, n, 0.0, line->epsilon, &anynul, &status)) { 
    printf("Failed to read column %d (Epsilon).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Epsilon_Err", &colnum, &status)) {
    printf("Could not determine column number for (Epsilon_Err).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_colnull_flt(fp, colnum, 1l, 1l, n, line->epsilon_err, nularray, &anynul, &status)) { 
    printf("Failed to read column %d (Epsilon_Err).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Element", &colnum, &status)) {
    printf("Could not determine column number for (Element).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, line->element, &anynul, &status)) { 
    printf("Failed to read column %d (Element).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Ion", &colnum, &status)) {
    printf("Could not determine column number for (Ion).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, line->ion, &anynul, &status)) { 
    printf("Failed to read column %d (Ion).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "UpperLev", &colnum, &status)) {
    printf("Could not determine column number for (UpperLev).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, line->upperlev, &anynul, &status)) { 
    printf("Failed to read column %d (UpperLev).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "LowerLev", &colnum, &status)) {
    printf("Could not determine column number for (LowerLev).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, line->lowerlev, &anynul, &status)) { 
    printf("Failed to read column %d (LowerLev).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/

  /*
    for(colnum=0;colnum<10;colnum++) {
    printf("%d %f %f %e %f %d %d %d %d\n", colnum,
    line->lambda[colnum],
    line->lambda_err[colnum],
    line->epsilon[colnum],
    line->epsilon_err[colnum],
    line->element[colnum],
    line->ion[colnum],
    line->upperlev[colnum],
    line->lowerlev[colnum]);
    }
  */

  free(nularray);
  
  return(0);

}


int read_coco_info(fitsfile *fp, Coco_Type *coco,long n) {


  int status=0, colnum=0, anynul=0;
  int i;
  long l;
  char *nularray;

  /* first allocate the correct amount of memory */
  /* we need to do this in 2 stages since the continuum and
     pseudo continuum are stored in arrays */

  coco->n = n;
  coco->z = (long *)malloc(sizeof(long)*n);
  coco->rmj = (long *)malloc(sizeof(long)*n);
  coco->n_cont = (long *)malloc(sizeof(long)*n);
  coco->e_cont = (float **)malloc(sizeof(float *)*n);
  coco->continuum = (float **)malloc(sizeof(float *)*n);
  coco->cont_err = (float **)malloc(sizeof(float *)*n);
  coco->n_pseudo = (long *)malloc(sizeof(long)*n);
  coco->e_pseudo = (float **)malloc(sizeof(float *)*n);
  coco->pseudo = (float **)malloc(sizeof(float *)*n);
  coco->pseudo_err = (float **)malloc(sizeof(float *)*n);
  nularray = (char *)malloc(sizeof(char)*n);

  /* Now get column positions and read their content */
  if (fits_get_colnum(fp, CASEINSEN, "Z", &colnum, &status)) {
    printf("Could not determine column number for (Z).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, coco->z, &anynul, &status)) { 
    printf("Failed to read column %d (Z).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "rmJ", &colnum, &status)) {
    printf("Could not determine column number for (rmJ).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, coco->rmj, &anynul, &status)) { 
    printf("Failed to read column %d (rmJ).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "N_Cont", &colnum, &status)) {
    printf("Could not determine column number for (N_Cont).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, coco->n_cont, &anynul, &status)) { 
    printf("Failed to read column %d (N_Cont).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "N_Pseudo", &colnum, &status)) {
    printf("Could not determine column number for (N_Pseudo).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp, colnum, 1l, 1l, n, 0l, coco->n_pseudo, &anynul, &status)) { 
    printf("Failed to read column %d (N_Pseudo).\n",colnum); 
    fits_close_file(fp, &status);
    return(-2);
  }

  /* Stage 2  memory allocation*/
  coco->max_cont = 0l;
  for (i = 0; i < n; i++) 
    if (coco->n_cont[i] >= coco->max_cont) 
      coco->max_cont = coco->n_cont[i];

  coco->max_pseudo = 0l;
  for (i = 0; i < n; i++) 
    if (coco->n_pseudo[i] >= coco->max_pseudo) 
      coco->max_pseudo = coco->n_pseudo[i];
  
  for (i=0;i<n;i++) {
    coco->e_cont[i] = (float *) malloc(coco->max_cont*sizeof(float));
    coco->continuum[i] = (float *) malloc(coco->max_cont*sizeof(float));
    coco->cont_err[i] = (float *) malloc(coco->max_cont*sizeof(float));
    coco->e_pseudo[i] = (float *) malloc(coco->max_pseudo*sizeof(float));
    coco->pseudo[i] = (float *) malloc(coco->max_pseudo*sizeof(float));
    coco->pseudo_err[i] = (float *) malloc(coco->max_pseudo*sizeof(float));
  }


  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "E_Cont", &colnum, &status)) {
    printf("Could not determine column number for (E_Cont).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_cont, 0.0, coco->e_cont[l], &anynul, &status)) { 
      printf("Failed to read column %d (E_Cont).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Continuum", &colnum, &status)) {
    printf("Could not determine column number for (Continuum).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_cont, 0.0, coco->continuum[l], &anynul, &status)) { 
      printf("Failed to read column %d (Continuum).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Cont_Err", &colnum, &status)) {
    printf("Could not determine column number for (Cont_Err).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_cont, 0.0, coco->cont_err[l], &anynul, &status)) { 
      printf("Failed to read column %d (Cont_Err).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "E_Pseudo", &colnum, &status)) {
    printf("Could not determine column number for (E_Pseudo).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_pseudo, 0.0, coco->e_pseudo[l], &anynul, &status)) { 
      printf("Failed to read column %d (E_Pseudo).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Pseudo", &colnum, &status)) {
    printf("Could not determine column number for (Pseudo).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_pseudo, 0.0, coco->pseudo[l], &anynul, &status)) { 
      printf("Failed to read column %d (Pseudo).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }
  /* -------------------------------------------*/
  if (fits_get_colnum(fp, CASEINSEN, "Pseudo_Err", &colnum, &status)) {
    printf("Could not determine column number for (Pseudo_Err).\n"); 
    fits_close_file(fp, &status);
    return(-2);
  }
  for (l=0l;l<n;l++) {
    if (fits_read_col_flt(fp, colnum, 1l+l, 1l, coco->max_pseudo, 0.0, coco->pseudo_err[l], &anynul, &status)) { 
      printf("Failed to read column %d (Pseudo_Err).\n",colnum); 
      fits_close_file(fp, &status);
      return(-2);
    }
  }

  /*
  for(colnum=0;colnum<10;colnum++) {
    printf("\n%d %d %d %d %d\n", colnum,
	   coco->z[colnum],
	   coco->rmj[colnum],
	   coco->n_cont[colnum],
	   coco->n_pseudo[colnum]);
    for (i=0;i<5;i++) 
      printf("%f  ",coco->e_cont[colnum][i]);
  }
  */  

  free(nularray);
  
  return(0);

}

/* In the following we will assume that the user just wants
   to read in the line and coco files. We can extent to cont,
   bincont, and xspec it needed */
   
int load_plasma(char *name, char *line_file, char *coco_file, char *file_stem, int verbose) {
  /* 
     have load_plasma.sl routine to provide c routine with correct input.
  */

  fitsfile *fp_line;
  fitsfile *fp_coco;
  int status = 0, hdutype=0, nkey = 0, i, keypos=0, keyexists=0,
    anynul=0, colnum=0, ivalue = 0;
  int  indx[4];
  long l;
  float fvalue = 0.0;
  int element, ion;
  char keyname[FLEN_KEYWORD], value[FLEN_KEYWORD], comment[FLEN_COMMENT];
  Apec_Model *am;

  am = get_new_apec_model_ptr();
  if (verbose == 1) {
    printf("Reading APEC model from file....\n");
    printf("Reading linelist %s\n",line_file);
  }

  /*                                             */
  /* open _line.fits file                        */
  /*                                             */
  if (fits_open_file(&fp_line, line_file, READONLY, &status)) { 
    printf("File %s does not exist.\n",line_file); 
    return(-1); }

  /* Copy file info to am */
  am->line_file = strndup(line_file,1024);
  am->file_stem = strndup(file_stem,1024);
  /* and assign it its name */
  /* RH, we should really check if this name is a unique name to make sure
     we can identify our models. Something to do later */
  am->model_name = strndup(name,1024);
  if (verbose == 1) printf("  Model name : %s\n", am->model_name);

  /*                                             */
  /* Read header info                            */
  /*                                             */
  if (fits_read_key_str(fp_line, "SIONBAL", (am->ionbal), comment, &status)) {
    printf("Cannot read KEYWORD SIONBAL.\n"); 
    fits_close_file(fp_line, &status);
    return(-2);   }
  if (verbose == 1) printf("  Ionization Balance file : %s\n", am->ionbal);

  if (fits_read_key_str(fp_line, "SFILEMAP", (am->filemap), comment, &status)) {
    printf("Cannot read KEYWORD SFILEMAP.\n"); 
    fits_close_file(fp_line, &status);
    return(-2);   }
  if (verbose == 1) printf("  FileMap : %s\n", am->filemap);

  if (fits_read_key_str(fp_line, "SABUND_SOURCE", (am->abundances), comment, &status)) {
    printf("Cannot read KEYWORD SABUND_SOURCE.\n"); 
    fits_close_file(fp_line, &status);
    return(-2);   }
  if (verbose == 1) printf("  Abundances from : %s\n", am->abundances);

  if (fits_read_keyword(fp_line, "NEI", value, comment, &status)) {
    if (verbose == 1) printf("  CIE calculation\n");
    am->nei = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  NEI calculation\n");
    am->nei = 1;
  }

  /*                                             */
  /* Get output options for apec run from header */
  /*                                             */
  if (fits_read_keyword(fp_line, "OUTPUT_LINE", value, comment, &status)) {
    printf("Cannot find keywords OUTPUT_LINE.\n"); 
    fits_close_file(fp_line, &status);
    return(-2);   
  } else {
    /* read next keyword SMODELNAME */
    if (fits_get_hdrpos(fp_line, &keyexists, &keypos, &status)) {
      printf("Cannot get header position.\n"); 
      fits_close_file(fp_line, &status);
      return(-2);   
    } else {
      if (fits_read_keyn(fp_line, keypos, keyname, (am->output_line), comment, &status)) {
	printf("Cannot read SMODELNAME.\n"); 
	fits_close_file(fp_line, &status);
	return(-2);   
      } else {
	if (strcmp(keyname,"SMODELNAME") != 0) {
	  strcpy(am->output_line,"");
	} else {
	  strtrim(am->output_line);
	}
      }
    }
  }


  /*                                             */
  /* Get physical processes from header          */
  /*                                             */
  if (fits_read_keyword(fp_line, "DO_RRC", value, comment, &status)) {
    if (verbose == 1) printf("  Did not calculate Radiative Recombination\n");
    am->do_rrc = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  Included Radiative Recombination\n");
    am->do_rrc = 1;
  }

  if (fits_read_keyword(fp_line, "DO_SATEL", value, comment, &status)) {
    if (verbose == 1) printf("  Did not calculate satellite lines\n");
    am->do_satel = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  Calculated satellite lines\n");
    am->do_satel = 1;
  }

  if (fits_read_keyword(fp_line, "DO_LINES", value, comment, &status)) {
    if (verbose == 1) printf("  Did not calculates lines\n");
    am->do_lines = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  Calculates lines\n");
    am->do_lines = 1;
  }

  if (fits_read_keyword(fp_line, "DO_TWOPH", value, comment, &status)) {
    if (verbose == 1) printf("  Did not included Two Photon processes\n");
    am->do_twoph = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  Included Two Photon processes\n");
    am->do_twoph = 1;
  }

  if (fits_read_keyword(fp_line, "DO_BREMS", value, comment, &status)) {
    if (verbose == 1) printf("  Did not included Bremsstrahlung\n");
    am->do_brems = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    if (verbose == 1) printf("  Calculated Bremsstrahlung of type ");
    am->do_brems = 1;
    if (fits_read_key_str(fp_line, "SBREMS_TYPE", (am->bremstype), comment, &status)) {
      printf("Cannot read KEYWORD SBREMS_TYPE.\n"); 
      fits_close_file(fp_line, &status);
      return(-2);       }
    if (verbose == 1) printf("%s\n",am->bremstype);
  }

  /*                                             */
  /* Get binning parameters from header          */
  /*                                             */
  am->bin_type = -1;
  /* energy/wavelength grid */
  if (! fits_read_keyword(fp_line, "BIN_LIN_ENERGY", value, comment, &status)) {
    am->bin_type = BIN_LIN_ENERGY; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "BIN_LOG_ENERGY", value, comment, &status)) {
    am->bin_type = BIN_LOG_ENERGY; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "BIN_LIN_WAVE", value, comment, &status)) {
    am->bin_type = BIN_LIN_WAVE; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "BIN_LOG_WAVE", value, comment, &status)) {
    am->bin_type = BIN_LOG_WAVE; } else { status = 0; }
  if (am->bin_type == -1) {
      printf("Cannot detemine BIN_TYPE.\n"); 
      fits_close_file(fp_line, &status);
  }

  if (verbose == 1) {
    printf("  Spectral binning is ");
    switch (am->bin_type) {
    case BIN_LIN_ENERGY : printf("linear in energy\n"); break;
    case BIN_LOG_ENERGY : printf("logarithmic in energy\n"); break;
    case BIN_LIN_WAVE : printf("linear in wavelength\n"); break;
    case BIN_LOG_WAVE : printf("logarithmic in wavelength\n"); break;
    }
  }


  if (am->bin_type == BIN_LIN_ENERGY || am->bin_type == BIN_LOG_ENERGY) {
    if (fits_read_key(fp_line, TINT, "INUM_E", &(am->num_bin), comment, &status)) {
      printf("Cannot read KEYWORD INUM_E.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (fits_read_key(fp_line, TFLOAT, "DE_START", &(am->bin_start), comment, &status)) {
      printf("Cannot read KEYWORD DE_START.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (fits_read_key(fp_line, TFLOAT, "DE_END", &(am->bin_end), comment, &status)) {
      printf("Cannot read KEYWORD DE_END.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (verbose == 1) {
      printf("    %d number of bins between %f and %f keV\n",am->num_bin,am->bin_start, am->bin_end);
    }
  }
  if (am->bin_type == BIN_LIN_WAVE || am->bin_type == BIN_LOG_WAVE) {
    if (fits_read_key(fp_line, TINT, "INUM_LAMBDA", &(am->num_bin), comment, &status)) {
      printf("Cannot read KEYWORD INUM_LAMBDA.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (fits_read_key(fp_line, TFLOAT, "DLAMBDA_START", &(am->bin_start), comment, &status)) {
      printf("Cannot read KEYWORD DLAMBDA_START.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (fits_read_key(fp_line, TFLOAT, "DLAMBDA_END", &(am->bin_end), comment, &status)) {
      printf("Cannot read KEYWORD DLAMBDA_END.\n"); 
      fits_close_file(fp_line, &status); 
      return(-2); }
    if (verbose == 1) {
      printf("    %d number of bins between %f and %f Angstrom\n",am->num_bin,am->bin_start, am->bin_end);
    }
  }
  
  if (fits_read_key(fp_line, TFLOAT, "DMIN_EPSILON", &(am->min_epsilon), comment, &status)) {
    printf("Cannot read KEYWORD DMIN_EPSILON.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
    if (verbose == 1) {
      printf("  Minimun emissivity to be included in linelist is %e\n",am->min_epsilon);
    }
  
  /*                                             */
  /* Get physical parameters from header         */
  /*                                             */
  am->lin_dens = -1;
  /* Density grid */
  if (! fits_read_keyword(fp_line, "LIN_DENS", value, comment, &status)) {
    am->lin_dens = LIN_DENS; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "LOG_DENS", value, comment, &status)) {
    am->lin_dens = LOG_DENS; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "ARB_DENS", value, comment, &status)) {
    am->lin_dens = ARB_DENS; } else { status = 0; }
  if (am->lin_dens == -1) {
      printf("Cannot detemine LIN_DENS.\n"); 
      fits_close_file(fp_line, &status);
      return(-3); 
  }
  if (verbose == 1) {
    printf("  Density runs ");
    switch (am->lin_dens) {
      case LIN_DENS : printf("linear "); break;
      case LOG_DENS : printf("logarithmic "); break;
      case ARB_DENS : printf("arbitrary "); break;
    }
  }

  if (fits_read_key(fp_line, TINT, "INUM_DENSITIES", &(am->num_dens), comment, &status)) {
    printf("Cannot read KEYWORD INUM_DENSITIES.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (fits_read_key(fp_line, TFLOAT, "DDENSITY_START", &(am->dens_start), comment, &status)) {
    printf("Cannot read KEYWORD DDENSITY_START.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (fits_read_key(fp_line, TFLOAT, "DDENSITY_STEP", &(am->dens_step), comment, &status)) {
    printf("Cannot read KEYWORD DDENSITY_STEP.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (verbose == 1) {
    printf("from %f to %f with stepsize %d for %f steps\n",am->dens_start,
	   am->dens_start + am->num_dens * am->dens_step,
	   am->num_dens, am->dens_step);
  }

  am->lin_temp = -1;
  /* Temperature grid */
  if (! fits_read_keyword(fp_line, "LIN_TEMP", value, comment, &status)) {
    am->lin_temp = LIN_TEMP; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "LOG_TEMP", value, comment, &status)) {
    am->lin_temp = LOG_TEMP; } else { status = 0; }
  if (! fits_read_keyword(fp_line, "ARB_TEMP", value, comment, &status)) {
    am->lin_temp = ARB_TEMP; } else { status = 0; }
  if (am->lin_temp == -1) {
      printf("Cannot detemine LIN_TEMP.\n"); 
      fits_close_file(fp_line, &status);
  }
  if (verbose == 1) {
    printf("  Temperature runs "); 
    switch (am->lin_temp) {
      case LIN_TEMP : printf("linear "); break;
      case LOG_TEMP : printf("logarithmic "); break;
      case ARB_TEMP : printf("arbitrary "); break;
    }
  }

  if (fits_read_key(fp_line, TINT, "INUM_TEMP", &(am->num_temp), comment, &status)) {
    printf("Cannot read KEYWORD INUM_TEMP.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (fits_read_key(fp_line, TFLOAT, "DTEMP_START", &(am->temp_start), comment, &status)) {
    printf("Cannot read KEYWORD DTEMP_START.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (fits_read_key(fp_line, TFLOAT, "DTEMP_STEP", &(am->temp_step), comment, &status)) {
    printf("Cannot read KEYWORD DTEMP_STEP.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2); }
  if (verbose == 1) {
    printf("from %f to %f with stepsize %d for %f steps\n",am->temp_start,
	   am->temp_start + am->num_temp * am->temp_step,
	   am->num_temp, am->temp_step);
  }

  /* Find included Atoms */
  if (fits_get_hdrspace(fp_line, &nkey, NULL, &status)) {
    printf("Cannot determine # of KEYWORDS.\n"); 
    fits_close_file(fp_line, &status); 
    return(-2);   }
    
  if (verbose == 1) printf("  Reading elements/ions included in calculation\n    ");
  for (i = 1; i < nkey; i++) {
    fits_read_keyn(fp_line, i, keyname, value, comment, &status);
    if (strcmp(keyname,"SATOM") == 0) {
      strtrim(value);
      if (verbose == 1) printf("%s, ",value);
      element = decode_element(value);
      ion = decode_ion(value);
      /* now flag correct ions */
      if (ion != 0) {
	am->atoms[get_ionpop_indx(element,ion)] = 1;
      } else {
	for (ion = 1; ion <= element+1; ion++) {
	  am->atoms[get_ionpop_indx(element,ion)] = 1;
	}
      }
    }
  }
  if (verbose == 1) printf("\n");
      

  /* move over to HDU #2. This contains the information about the 
     model components */

  if (fits_movabs_hdu(fp_line, 2, &hdutype, &status)) { 
    printf("Failed to reach HDU #2 of file %s.\n",line_file); 
    fits_close_file(fp_line, &status);
    return(-10);
  }

  /* the line table have 4 columns. Allocate room ad read columns */
  
  /* set number of components equal to either nT*nrho or nT in the
     case of an NEI calculation */
  if (am->nei) {
    if (am->num_dens == am->num_temp) {
      am->n = am->num_dens;
    } else {
      printf("# of density components and temperature components is not\nthe same for this NEI calculation. This is suspect.\n");
      fits_close_file(fp_line, &status);
      return(-10);
    }
  } else {
    am->n = am->num_dens * am->num_temp;
  }

  /* allocate memory */
  am->kT = (float *)malloc(sizeof(float)*am->n);
  am->density = (float *)malloc(sizeof(float)*am->n);
  am->Nelement = (long *)malloc(sizeof(long)*am->n);
  am->Nline = (long *)malloc(sizeof(long)*am->n);

  if (fits_get_colnum(fp_line, CASEINSEN, "kT", &colnum, &status)) {
    printf("Could not determine column number for (kT) of file %s.\n",line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  if (fits_read_col_flt(fp_line, colnum, 1l, 1l, am->n, 0.0, am->kT, &anynul, &status)) { 
    printf("Failed to read column %d (kT) of file %s.\n",colnum,line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  /* Find minumum and maximum temperature */
  am->Tmin = am->kT[0];
  am->Tmax = am->kT[0];
  for (l=0; l<am->n; l++) {
    if (am->kT[l] < am->Tmin) am->Tmin = am->kT[l];
    if (am->kT[l] > am->Tmax) am->Tmax = am->kT[l];
  }

  if (fits_get_colnum(fp_line, CASEINSEN, "EDensity", &colnum, &status)) {
    printf("Could not determine column number for (kT) of file %s.\n",line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  if (fits_read_col_flt(fp_line, colnum, 1l, 1l, am->n, 0.0, am->density, &anynul, &status)) { 
    printf("Failed to read column %d (density) of file %s.\n",colnum,line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  /* Find minumum and maximum density */
  am->densitymin = am->density[0];
  am->densitymax = am->density[0];
  for (l=0; l<am->n; l++) {
    if (am->density[l] < am->densitymin) am->densitymin = am->density[l];
    if (am->density[l] > am->densitymax) am->densitymax = am->density[l];
  }

  if (fits_get_colnum(fp_line, CASEINSEN, "Nelement", &colnum, &status)) {
    printf("Could not determine column number for (kT) of file %s.\n",line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp_line, colnum, 1l, 1l, am->n, 0l, am->Nelement, &anynul, &status)) { 
    printf("Failed to read column %d (Nelement) of file %s.\n",colnum,line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  if (fits_get_colnum(fp_line, CASEINSEN, "Nline", &colnum, &status)) {
    printf("Could not determine column number for (kT) of file %s.\n",line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp_line, colnum, 1l, 1l, am->n, 0l, am->Nline, &anynul, &status)) { 
    printf("Failed to read column %d (Nline) of file %s.\n",colnum,line_file); 
    fits_close_file(fp_line, &status);
    return(-2);
  }

  /* before reading in the separate line HDUs we will construct a 
     2D array holding the temperature and density grid. This will
     help enormously in the subsequent spectrum reconstruction */

  setup_kT_density_grid(am); 


  /* allocate correct number of line structure */
  am->line = (Line_Type *)malloc(am->n*sizeof(Line_Type));

  /* loop over hdu's and read in lines */
  am->Nline_total = 0;
  for (l = 0; l<am->n; l++) {
    
    am->Nline_total += am->Nline[l];
    (am->line[l]).kT = am->kT[l];
    (am->line[l]).density = am->density[l];
    if (l == 0) {
      (am->line[l]).linelist_start = 0;
    } else {
      (am->line[l]).linelist_start = (am->line[l-1]).linelist_start + 
	                              am->Nline[l-1];
    }

    if (fits_movrel_hdu(fp_line, 1, &hdutype, &status)) { 
      printf("Failed to move to next HDU of file %s.\n",line_file); 
      fits_close_file(fp_line, &status);
      return(-10);
    }

    read_line_info(fp_line,&(am->line[l]),am->Nline[l]);

  }


  /* close _line.fits file */
  if (fits_close_file(fp_line, &status)) {
    printf("Failed to close file %s.\n",line_file);
  }

  /* ---------------------------------------------------------------- */
  /* ---------------------------------------------------------------- */
  /* ---------------------------------------------------------------- */
  /* ---------------------------------------------------------------- */
  /* ---------------------------------------------------------------- */
  /* For the coco file we need to read the header file and make sure  */
  /* the parameters are compatible with the line file so that we do   */
  /* not combine incommesurable datasets                              */
  /* ---------------------------------------------------------------- */

  if (verbose == 1) {
    printf("Reading continuum %s\n",coco_file);
  }
  /* open _coco.fits file */
  if (fits_open_file(&fp_coco, coco_file, READONLY, &status)) {
    printf("File %s does not exist.\n",coco_file);
    return(-1);
  }
  /*                                             */
  am->coco_file = strndup(coco_file,1024);
  if (fits_read_key_str(fp_coco, "SIONBAL", value, comment, &status)) {
    printf("Cannot read KEYWORD SIONBAL.\n"); 
    fits_close_file(fp_coco, &status);
    return(-2);   }
  if (strcmp(value,am->ionbal) != 0) {
    printf("KEYWORD SIONBAL in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_key_str(fp_coco, "SFILEMAP", value, comment, &status)) {
    printf("Cannot read KEYWORD SFILEMAP.\n"); 
    fits_close_file(fp_coco, &status);
    return(-2);   }
  if (strcmp(value,am->filemap) != 0) {
    printf("KEYWORD SFILEMAP in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_key_str(fp_coco, "SABUND_SOURCE", value, comment, &status)) {
    printf("Cannot read KEYWORD SABUND_SOURCE.\n"); 
    fits_close_file(fp_coco, &status);
    return(-2);   }
  if (strcmp(value,am->abundances) != 0) {
    printf("KEYWORD SABUND_SOURCE in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_keyword(fp_coco, "NEI", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
  }
  if (ivalue != am->nei) {
    printf("KEYWORD NEI in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
 
  /*                                             */
  /* Get output options for apec run from header */
  /*                                             */

  if (fits_read_keyword(fp_coco, "OUTPUT_COCO", value, comment, &status)) {
    printf("Cannot find keyword OUTPUT_COCO.\n"); 
    fits_close_file(fp_coco, &status);
    return(-2);   
  } else {
    /* read next keyword SMODELNAME */
    if (fits_get_hdrpos(fp_coco, &keyexists, &keypos, &status)) {
      printf("Cannot get header position.\n"); 
      fits_close_file(fp_coco, &status);
      return(-2);   
    } else {
      if (fits_read_keyn(fp_coco, keypos, keyname, (am->output_coco), comment, &status)) {
	printf("Cannot read SMODELNAME.\n"); 
	fits_close_file(fp_coco, &status);
	return(-2);   
      } else {
	if (strcmp(keyname,"SMODELNAME") != 0) {
	  strcpy(am->output_coco,"");
	} else {
	  strtrim(am->output_coco);
	}
      }
    }
  }
  if (strcmp(am->output_line,am->output_line) != 0) {
    printf("KEYWORD SMODELNAME in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }


  /*                                             */
  /* Get physical processes from header          */
  /*                                             */
  if (fits_read_keyword(fp_coco, "DO_RRC", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
  }
  if (ivalue != am->do_rrc) {
    printf("KEYWORD DO_RRC/NO_RRC in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_keyword(fp_coco, "DO_SATEL", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
  }
  if (ivalue != am->do_satel) {
    printf("KEYWORD DO_SATEL/NO_SATEL in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_keyword(fp_coco, "DO_LINES", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
  }
  if (ivalue != am->do_lines) {
    printf("KEYWORD DO_LINES/NO_LINES in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_keyword(fp_coco, "DO_TWOPH", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
  }
  if (ivalue != am->do_twoph) {
    printf("KEYWORD DO_TWOPH/NO_TWOPH in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_keyword(fp_coco, "DO_BREMS", value, comment, &status)) {
    ivalue = 0;
    status = 0;       /* reset status so not to confuse subsequent calls */
  } else {
    ivalue = 1;
    if (fits_read_key_str(fp_coco, "SBREMS_TYPE", value, comment, &status)) {
      printf("Cannot read KEYWORD SBREMS_TYPE.\n"); 
      fits_close_file(fp_coco, &status);
      return(-2);       }
  }
  if (ivalue != am->do_brems) {
    printf("KEYWORD DO_BREMS/NO_BREMS in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  if (strcmp(value,am->bremstype) != 0) {
    printf("KEYWORD SBREMS_TYPE in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }


  /*                                             */
  /* Get binning parameters from header          */
  /*                                             */
  ivalue = -1;
  /* energy/wavelength grid */
  if (! fits_read_keyword(fp_coco, "BIN_LIN_ENERGY", value, comment, &status)) {
    ivalue = BIN_LIN_ENERGY; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "BIN_LOG_ENERGY", value, comment, &status)) {
    ivalue = BIN_LOG_ENERGY; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "BIN_LIN_WAVE", value, comment, &status)) {
    ivalue = BIN_LIN_WAVE; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "BIN_LOG_WAVE", value, comment, &status)) {
    ivalue = BIN_LOG_WAVE; } else { status = 0; }
  if (ivalue == -1) {
      printf("Cannot detemine BIN_TYPE.\n"); 
      fits_close_file(fp_coco, &status);
  }
  if (ivalue != am->bin_type) {
    printf("BIN_TYPE in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (am->bin_type == BIN_LIN_ENERGY || am->bin_type == BIN_LOG_ENERGY) {
    if (fits_read_key(fp_coco, TINT, "INUM_E", &ivalue, comment, &status)) {
      printf("Cannot read KEYWORD INUM_E.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (ivalue != am->num_bin) {
      printf("INUM_E in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
    if (fits_read_key(fp_coco, TFLOAT, "DE_START", &fvalue, comment, &status)) {
      printf("Cannot read KEYWORD DE_START.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (fvalue != am->bin_start) {
      printf("DE_START in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
    if (fits_read_key(fp_coco, TFLOAT, "DE_END", &fvalue, comment, &status)) {
      printf("Cannot read KEYWORD DE_END.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (fvalue != am->bin_end) {
      printf("DE_END in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
  }
  if (am->bin_type == BIN_LIN_WAVE || am->bin_type == BIN_LOG_WAVE) {
    if (fits_read_key(fp_coco, TINT, "INUM_LAMBDA", &ivalue, comment, &status)) {
      printf("Cannot read KEYWORD INUM_LAMBDA.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (ivalue != am->num_bin) {
      printf("INUM_LAMBDA in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
    if (fits_read_key(fp_coco, TFLOAT, "DLAMBDA_START", &fvalue, comment, &status)) {
      printf("Cannot read KEYWORD DLAMBDA_START.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (fvalue != am->bin_start) {
      printf("DLAMBDA_START in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
    if (fits_read_key(fp_coco, TFLOAT, "DLAMBDA_END", &fvalue, comment, &status)) {
      printf("Cannot read KEYWORD DLAMBDA_END.\n"); 
      fits_close_file(fp_coco, &status); 
      return(-2); }
    if (fvalue != am->bin_end) {
      printf("DLAMBDA_END in _line and _coco are not identical.\n"); 
      fits_close_file(fp_coco, &status);
      return(-4);   
    }
  }
  
  if (fits_read_key(fp_coco, TFLOAT, "DMIN_EPSILON", &fvalue, comment, &status)) {
    printf("Cannot read KEYWORD DMIN_EPSILON.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (fvalue != am->min_epsilon) {
    printf("DMIN_EPSILON in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  
  /*                                             */
  /* Get physical parameters from header         */
  /*                                             */
  ivalue = -1;
  /* Density grid */
  if (! fits_read_keyword(fp_coco, "LIN_DENS", value, comment, &status)) {
    ivalue = LIN_DENS; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "LOG_DENS", value, comment, &status)) {
    ivalue = LOG_DENS; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "ARB_DENS", value, comment, &status)) {
    ivalue = ARB_DENS; } else { status = 0; }
  if (ivalue == -1) {
      printf("Cannot detemine LIN_DENS.\n"); 
      fits_close_file(fp_coco, &status);
      return(-3); 
  }
  if (ivalue != am->lin_dens) {
    printf("DENSITY SPACING in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_key(fp_coco, TINT, "INUM_DENSITIES", &ivalue, comment, &status)) {
    printf("Cannot read KEYWORD INUM_DENSITIES.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (ivalue != am->num_dens) {
    printf("INUM_DENSITIES in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  if (fits_read_key(fp_coco, TFLOAT, "DDENSITY_START", &fvalue, comment, &status)) {
    printf("Cannot read KEYWORD DDENSITY_START.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (fvalue != am->dens_start) {
    printf("DDENSITY_START in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  if (fits_read_key(fp_coco, TFLOAT, "DDENSITY_STEP", &fvalue, comment, &status)) {
    printf("Cannot read KEYWORD DDENSITY_STEP.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (fvalue != am->dens_step) {
    printf("IDENSITY_STEP in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  ivalue = -1;
  /* Temperature grid */
  if (! fits_read_keyword(fp_coco, "LIN_TEMP", value, comment, &status)) {
    ivalue = LIN_TEMP; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "LOG_TEMP", value, comment, &status)) {
    ivalue = LOG_TEMP; } else { status = 0; }
  if (! fits_read_keyword(fp_coco, "ARB_TEMP", value, comment, &status)) {
    ivalue = ARB_TEMP; } else { status = 0; }
  if (ivalue == -1) {
      printf("Cannot detemine LIN_TEMP.\n"); 
      fits_close_file(fp_coco, &status);
  }
  if (ivalue != am->lin_temp) {
    printf("TEMPERATURE SPACING in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  if (fits_read_key(fp_coco, TINT, "INUM_TEMP", &ivalue, comment, &status)) {
    printf("Cannot read KEYWORD INUM_TEMP.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (ivalue != am->num_temp) {
    printf("INUM_TEMP in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  if (fits_read_key(fp_coco, TFLOAT, "DTEMP_START", &fvalue, comment, &status)) {
    printf("Cannot read KEYWORD DTEMP_START.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (fvalue != am->temp_start) {
    printf("DTEMP_START in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }
  if (fits_read_key(fp_coco, TFLOAT, "DTEMP_STEP", &fvalue, comment, &status)) {
    printf("Cannot read KEYWORD DTEMP_STEP.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2); }
  if (fvalue != am->temp_step) {
    printf("DTEMP_STEP in _line and _coco are not identical.\n"); 
    fits_close_file(fp_coco, &status);
    return(-4);   
  }

  /* Find included Atoms */
  if (fits_get_hdrspace(fp_coco, &nkey, NULL, &status)) {
    printf("Cannot determine # of KEYWORDS.\n"); 
    fits_close_file(fp_coco, &status); 
    return(-2);   }

    
  for (i = 1; i < nkey; i++) {
    fits_read_keyn(fp_coco, i, keyname, value, comment, &status);
    if (strcmp(keyname,"SATOM") == 0) {
      strtrim(value);
      element = decode_element(value);
      ion = decode_ion(value);
      /* now flag correct ions */
      if (ion != 0) {
	if (am->atoms[get_ionpop_indx(element,ion)] == 0) {
	  printf("Ion %s not included in _line.\n",value); 
	  fits_close_file(fp_coco, &status);
	  return(-4);   
	}
      } else {
	for (ion = 1; ion <= element+1; ion++) {
	  if (am->atoms[get_ionpop_indx(element,ion)] == 0) {
	    printf("Element %s not included in _line.\n",value); 
	    fits_close_file(fp_coco, &status);
	    return(-4);   
	  }
	}
      }
    }
  }


  /* move over to HDU #2. This contains the information about the 
     model components. Assuming that we passed the above test we
     can be sure that the infomation HDU2 is the same as in line.
     So we onlu need to read two more columns: NCont NPseudo */

  if (fits_movabs_hdu(fp_coco, 2, &hdutype, &status)) { 
    printf("Failed to reach HDU #2 of file %s.\n",line_file); 
    fits_close_file(fp_coco, &status);
    return(-10);
  }

  /* the coco table have 5 columns. Allocate room and read 2 columns columns */
  
  /* allocate memory */
  am->Ncont = (long *)malloc(sizeof(long)*am->n);
  am->Npseudo = (long *)malloc(sizeof(long)*am->n);

  if (fits_get_colnum(fp_coco, CASEINSEN, "NCont", &colnum, &status)) {
    printf("Could not determine column number for (NCont) of file %s.\n",coco_file); 
    fits_close_file(fp_coco, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp_coco, colnum, 1l, 1l, am->n, 0l, am->Ncont, &anynul, &status)) { 
    printf("Failed to read column %d (NCont) of file %s.\n",colnum,coco_file); 
    fits_close_file(fp_coco, &status);
    return(-2);
  }
  if (fits_get_colnum(fp_coco, CASEINSEN, "NPseudo", &colnum, &status)) {
    printf("Could not determine column number for (NPseudo) of file %s.\n",coco_file); 
    fits_close_file(fp_coco, &status);
    return(-2);
  }
  if (fits_read_col_lng(fp_coco, colnum, 1l, 1l, am->n, 0l, am->Npseudo, &anynul, &status)) { 
    printf("Failed to read column %d (NPseudo) of file %s.\n",colnum,coco_file); 
    fits_close_file(fp_coco, &status);
    return(-2);
  }

  /* allocate correct number of line structure */
  am->coco = (Coco_Type *)malloc(am->n*sizeof(Coco_Type));

  /* loop over hdu's and read in lines */
  for (l = 0; l<am->n; l++) {
    
    (am->coco[l]).kT = am->kT[l];
    (am->coco[l]).density = am->density[l];

    if (fits_movrel_hdu(fp_coco, 1, &hdutype, &status)) { 
      printf("Failed to move to next HDU of file %s.\n",coco_file); 
      fits_close_file(fp_coco, &status);
      return(-10);
    }

    read_coco_info(fp_coco,&(am->coco[l]),am->Nelement[l]);

  }


  /* close _coco.fits file */
  if (fits_close_file(fp_coco, &status)) {
    printf("Failed to close file %s.\n",coco_file);
  }

  /*find_grid_indices(am,1.0,1.0,indx);

  for(i=0;i<4;i++) printf("%d ",indx[i]);
  printf("\n");
  printf("%f %f %f %f\n",am->grid_kT[indx[0]],am->grid_kT[indx[1]],am->grid_density[indx[2]],am->grid_density[indx[3]]);*/

  /* 
     Determine some extraneous characteristics of the model 
  */

  /* 1. min and max wavelength */
  /* 2. min and max emissivity */
  /* 3. min and max kev */
  am->wvl_min =  1.0e30;
  am->wvl_max = -1.0e30;
  for (i=0;i<am->n;i++) {
    for (l=0;l<(am->line[i]).n;l++) {
      if ((am->line[i]).lambda[l] < am->wvl_min) 
	am->wvl_min = (am->line[i]).lambda[l];
      if ((am->line[i]).lambda[l] > am->wvl_max) 
	am->wvl_max = (am->line[i]).lambda[l];
      if ((am->line[i]).epsilon[l] < am->epsilon_min) 
	am->epsilon_min = (am->line[i]).epsilon[l];
      if ((am->line[i]).epsilon[l] > am->epsilon_max) 
	am->epsilon_max = (am->line[i]).epsilon[l];
    }
  }
  am->kev_min = HC_KEV / am->wvl_max;
  am->kev_max = HC_KEV / am->wvl_min;

  return(0);

}

int main() {

  int i, *indx=NULL;
  float binlo[100], binhi[100], spec[100], lines[100], coco[100];
  Apec_Model *am;
  long nelem, elem[1], nion, ion[1], nout=0, *out, *list_all, *list_comp;
  Emissivity_Curve_Type *ec;

  show_apec_models();
  i =   load_plasma("apec",
		    "/data/rsmith4/ronnieh/local/ciao3.1/ATOMDB/apec_line.fits",
		    "/data/rsmith4/ronnieh/local/ciao3.1/ATOMDB/apec_coco.fits",
		    "/data/rsmith4/ronnieh/local/ciao3.1/ATOMDB/apec_",1);
  show_apec_models();

  am = get_apec_model_with_index(1);

  for (i=0;i<100;i++) {
    binlo[i] = 10.0 + 0.02*i;
    binhi[i] = binlo[i]+0.02;
    lines[i] = 0.0;
    spec[i] = 0.0;
    coco[i] = 0.0;
  }
  
  nelem = 1; elem[0] = 10;
  nion = 0; ion[0] = 20;
  nout = 0; out = NULL;
  out = _elem_ion_("apec",nelem,elem,nion,ion,&nout); 
  list_all = _make_linelist_(nout,out,&nelem);

  printf("made linelist\n");
  nelem = 0; /* cancels the linelist (for testing purposes) */
  
  /* use solar abundances */
  /* extract_line_spectrum(am,1.0,1.0,1.0,solar_abundances,0.0,0,nelem,list_all,100,binlo,binhi,lines,indx);
     extract_coco_spectrum(am,1.0,1.0,1.0,solar_abundances,0,100,binlo,binhi,coco,indx); 
  */
  /* extract_spectrum(am,1.0,1.0,1.0,solar_abundances,0.0,LINES_AND_CONTINUUM,0,nelem,list_all,100,binlo,binhi,spec);
     if(indx != NULL) free(indx);*/

  printf("doing powerlaw\n");
  _em_power_law_("apec",1.0,1.0,10,0.1,10.0,0,1.0,solar_abundances,0.0,LINES_AND_CONTINUUM,0,nelem,list_all,100,binlo,binhi,spec);
  printf("done\n");

  /*
    for (i=0;i<100;i++) {
    printf("%8.3f  %8.3f  %20.10e + %20.10e = %20.10e\n",binlo[i],binhi[i],lines[i],coco[i],spec[i]);
    }
  */

  ec = emissivity_curve(am,8,7,1,7);
  /* for (i=0;i<ec->nT;i++) printf("%f %e\n",ec->kT[i],
     ec->emissivity[i][0]); */
  free_emissivity_curve(ec);


  free(out);
  free(list_all);

  free_all_apec_models(); 

  exit(0);

}
