#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>
#include <fitsio.h>

#include "apec.h"
#include "_apec_slang.h"

Apec_Model *models;
int nModel = 0;
int iModel = 0; 

void free_line(Line_Type line) {

  /* give memory occupied by line free */
  
  free(line.lambda);
  free(line.lambda_err);
  free(line.epsilon);
  free(line.epsilon_err);
  free(line.element);
  free(line.ion);
  free(line.upperlev);
  free(line.lowerlev);

}

void free_coco(Coco_Type coco) {

  int i;
  /* give memory occupied by coco free */

  for (i=0;i<coco.n;i++) {
    free(coco.e_cont[i]);
    free(coco.continuum[i]);
    free(coco.cont_err[i]);
    free(coco.e_pseudo[i]);
    free(coco.pseudo[i]);
    free(coco.pseudo_err[i]);
  }

  free(coco.z);
  free(coco.rmj);
  free(coco.n_cont);
  free(coco.e_cont);
  free(coco.continuum);
  free(coco.cont_err);
  free(coco.n_pseudo);
  free(coco.e_pseudo);
  free(coco.pseudo);
  free(coco.pseudo_err);

}

void free_apec_model(Apec_Model *am) {

  /* give memory occupied by am free */
  
  int i;

  for (i =0; i < am->n; i++) free_line(am->line[i]);
  for (i =0; i < am->n; i++) free_coco(am->coco[i]);
  free(am->line);
  free(am->coco);

  for (i =0; i < am->num_temp; i++) free(am->grid_kT_density[i]);
  free(am->grid_kT_density);
  free(am->grid_kT);
  free(am->grid_density);

  free(am->kT);
  free(am->density);
  free(am->Nelement);
  free(am->Nline);
  
  free(am->file_stem);
 
  free(am);
 
}

void free_all_apec_models() {

  Apec_Model *tmp;

  tmp = models;
  while (tmp->next != NULL) {
    tmp = tmp->next;
    free_apec_model(models);
    models = tmp;
  }

  free_apec_model(tmp);
  nModel = 0;
  
}

Apec_Model *get_apec_model_with_index(int index) {

  /* Return a pointer to the Apec model for which model_index 
     equals index
  */

  Apec_Model *tmp;
  
  if (nModel == 0) return(NULL);

  tmp = models;

  while (tmp->model_index != index && tmp->next != NULL) 
    tmp= tmp->next;

  if (tmp->model_index != index) {
    tmp = NULL;
    printf("Could not find Apec_Model with index %d\n",index);
  }

  return(tmp);

}

Apec_Model *get_apec_model_with_name(char *name) {

  /* Return a pointer to the Apec model for which model_name 
     equals name
  */

  Apec_Model *tmp;
  
  if (nModel == 0) return(NULL);

  tmp = models;

  while (strcmp(tmp->model_name,name)!=0 && tmp->next != NULL) 
    tmp= tmp->next;

  if (strcmp(tmp->model_name,name)!=0) {
    tmp = NULL;
    printf("Could not find Apec_Model with name %s\n",name);
  }

  return(tmp);

}


Apec_Model *get_new_apec_model_ptr() {
  
  Apec_Model *tmp;

  if (nModel == 0) 
    {
      /* i.e., this is the first model to be defined 
	 so allocate the memory and pass the pointer 
	 back */
      models = (Apec_Model *)malloc(sizeof(Apec_Model));
      if (models == NULL) 
	{
	  printf("Failed to allocate memory for new Apec_Model\n");
	  exit(-1);	
	}
      
      /* set next pointer to NULL to prevent problems with it
	 being undefined */
      models->next = NULL;
      tmp = models;
      nModel++;
      iModel = 0;
    } 
  else 
    {
      /* scroll to last defined model */
      tmp = models;
      iModel = 0;
      while (tmp->next != NULL) {
	tmp = tmp->next;
	iModel++;
      }
      
      /* now allocate and add a new model */
      tmp->next = (Apec_Model *)malloc(sizeof(Apec_Model));
      if (tmp->next == NULL) 
	{
	  printf("Failed to allocate memory for new Apec_Model\n");
	  exit(-1);	
	}
      tmp = tmp->next;
      tmp->next = NULL;    /* see comment above */
      nModel++;
      iModel++;
    }
  tmp->model_index = nModel;
	     
  return(tmp);

}


void show_apec_models() {

  Apec_Model *tmp;

  if (nModel == 0) {
    printf("No APEC model has been defined.\n");
  } else {
    tmp = models;
    printf("APEC models in memory...\n");
    while (tmp->next != NULL) {
      printf("  File stem : %s, index : %d\n",tmp->file_stem, tmp->model_index);
      tmp = tmp->next;
    }
    printf("  File stem : %s, index : %d\n",tmp->file_stem, tmp->model_index);
  }

}
