
%
% Small routine to load the default apec model 
%
%
define load_default_apecmodel() { 

   variable path, line, coco, stem;
   

   path = getenv("ATOMDB");
   if (path == NULL) {
      vmessage("ATOMDB apparently not installed; ATOMDB undefined");
      vmessage("Did not load the default apec model, please use the");
      vmessage("function load_apec_model(...) to load your favorite");
      vmessage("model.");
      return;
   }

   line = path + "/apec_line.fits";
   coco = path + "/apec_coco.fits";
   stem = path + "/apec_";

   load_apec_model("apec",line,coco,stem);

   return;

}


define load_apecmodel() { 
   %
   % Arguments are either 2 (name,stem)
   % or 
   %                      3 (name,line,coco)
   %
   variable name, line, coco, stem, istem;
   
   if ((_NARGS != 2) and (_NARGS != 3)) {
      usage("load_apec_model(String_Type modelname, String_Type filestem) or \nload_apec_model(String_Type modelname, String_Type linefilename, String_Type cocofilename).");
      _pop_n(_NARGS);
      return;
   }

   if (_NARGS == 2) {
      (name, stem) = ();
      line = stem + "line.fits";
      coco = stem + "coco.fits";
   } else {
      (name, line, coco) = ();
      istem = is_substr(line,"line.fits");
      if (istem != 0) {
	 stem = substr(line, 1, istem-1);
      } else {
	 stem = "not_available";
      }
   }

   % Should we make sure, here at this point that the files
   % actually exist? Maybe that would be the smart thing to do.

   if (0 == probe_file(line)) {
      vmessage("File %s does not exist.",line);
      return;
   }

   if (0 == probe_file(coco)) {
      vmessage("File %s does not exist.",coco);
      return;
   }
   
   %
   % Load the model
   load_apec_model(name,replace_envs(line),replace_envs(coco),replace_envs(stem));

   return;

}
