%
%
%
%

%
% little auxiliary routine to get the abundance into the correc
% array.
%
define make_abundance_array(ab_in) {

   %
   % The ab_in array holds the abundances from the apec model
   % in the order of increasing Z. We need to reorder them and 
   % put them in ab_out where Z is the array index.
   %
   
   % aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi
   variable indx = [2,6,7,8,9,10,12,13,14,16,18,20,26,28];
   variable i, n = 14;
   variable ab_out = Double_Type[100];
   
   ab_out[indx] = ab_in;

   % important! the Hydrogen abundance is always 1
   ab_out[1] = 1.0;


   return(ab_out);

}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec () {

   % input variables 
   variable kT, density, norm;                                     % 3
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 23
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable i, nargs = 23, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (kT,density,norm) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (kT,density,norm) = ();
      nbin = length(binlo);
   }


   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = plasma(modelname,spectrum_type,_apec_stuff_->linelist,
		     kT,density,norm,abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}


% Temporary fixes
define apec_cont() {
   
   variable nargs = 23;
   variable arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
            arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
            arg21,arg22, arg23;
   variable spectrum;

   if (_NARGS == nargs-1) {
      (arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
       arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
       arg21,arg22) = ();
      spectrum = apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
		      arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,
		      arg20,arg21,arg22);
   } else if (_NARGS == nargs) {
      (arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
       arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
       arg21,arg22,arg23) = ();
      spectrum = apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
		      arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,
		      arg20,arg21,arg22,arg23);

   } else {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }
   
   return(spectrum);
}

define apec_lines() {
   
   variable nargs = 23;
   variable arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
            arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
            arg21,arg22, arg23;
   variable spectrum;

   if (_NARGS == nargs-1) {
      (arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
       arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
       arg21,arg22) = ();
      spectrum = apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
		      arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,
		      arg20,arg21,arg22);
   } else if (_NARGS == nargs) {
      (arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
       arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
       arg21,arg22,arg23) = ();
      spectrum = apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
		      arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,
		      arg20,arg21,arg22,arg23);

   } else {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }


   return(spectrum);
}



%
%
%
% Register the different models
%
%
%() = register_model(
%"apec",                                             % modelname
%["kT","density","norm",                             % parameter names
% "abundance_He","abundance_C","abundance_N","abundance_O","abundance_F",
% "abundance_Ne","abundance_Mg","abundance_Al","abundance_Si",
% "abundance_S","abundance_Ar","abundance_Ca","abundance_Fe","abundance_Ni",
% "redshift","modelname","spectrum_type","linelist"],
%1,                                                  % 1 dimensional model
%[1.0, 1.0, 1.0,1.0,1.0,1.0,1.0,1.0,1.0,           
% 1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,0.0],              % default values
%[0.01,0.01,0.0,0.0,0.0,0.0,0.0,0.0,0.0,           
% 0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],              % minimum values
%[80.0,20.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,
% 100.0,100.0,100.0,100.0,100.0,100.0,100.0,100.0,10.0],  % maximum values
%[1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0],
%["apec","lines_and_continuum","all"]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec_simple () {

   % input variables 
   variable kT, density, norm, abundance;                          % 4
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type;                          % 2
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 9
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable abundances = Double_Type[30];
   variable i, nargs = 9, inc, nbin, integrated;
   variable abundances_array, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (kT,density,norm,abundance,redshift,modelname,str_spectrum_type,binlo) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (kT,density,norm,abundance,redshift,modelname,str_spectrum_type,binlo,binhi) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

   if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   %
   % Fix abundace array
   %
   abundances[*] = abundance;


   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = plasma(modelname,spectrum_type,NULL,
		     kT,density,norm,abundances,redshift,
		     keV,binlo,binhi);

   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }
   
   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}


%
%
%
% Register the different models
%
%
%() = register_model(
%"apec_simple",                                      % modelname
%["kT","density","norm","abundance",                 % parameter names
% "redshift","modelname","spectrum_type"],
%1,                                                  % 1 dimensional model
%[1.0, 1.0, 1.0,1.0,0.0],                            % default values
%[0.01,0.01,0.0,0.0,0.0],                            % minimum values
%[80.0,20.0,100.0,100.0,10.0],                       % maximum values
%[1,0,1,1,0],
%["apec","lines_and_continuum"]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec_nt () {

   % input variables 
   variable nfit;                                                  % 1
   variable kT, density, norm;                                     % 100+1+100
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 222
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable kT_and_norm, ncomp = 100, i_nfit;
   variable i, nargs = 22+2*ncomp, inc, nbin, integrated, value;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      kT_and_norm = array_from_args(__pop_args(2*ncomp));
      (nfit,density) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      kT_and_norm = array_from_args(__pop_args(2*ncomp));
      (nfit,density) = ();
      nbin = length(binlo);
   }
   i_nfit = typecast(nfit,Integer_Type);
   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

   if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   %
   % Fix kT and Norm arrays
   %
   kT = Double_Type[i_nfit];
   norm = Double_Type[i_nfit];
   for(i=0;i<i_nfit;i++) {
      kT[i] = kT_and_norm[2*i];
      norm[i] = kT_and_norm[2*i+1];
   }
   value = density;
   density = Double_Type[i_nfit];
   density[*] = value;

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %


   spectrum = ncomp_plasma(modelname,spectrum_type,_apec_stuff_->linelist,
   			   kT,density,norm,abundances,redshift,
   			   keV,binlo,binhi);
   
   if (spectrum == NULL) {
      spectrum = @binlo;
   }

   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec_ndensity () {

   % input variables 
   variable nfit;                                                  % 1
   variable kT, density, norm;                                     % 1+100+100
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 222
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable density_and_norm, ncomp = 100;
   variable i, nargs = 22+2*ncomp, inc, nbin, integrated, value;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      density_and_norm = array_from_args(__pop_args(2*ncomp));
      (nfit,kT) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      density_and_norm = array_from_args(__pop_args(2*ncomp));
      (nfit,kT) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

   if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   %
   % Fix kT and Norm arrays
   %
   density = Double_Type[nfit];
   norm = Double_Type[nfit];
   for(i=0;i<nfit;i++) {
      density[i] = density_and_norm[2*i];
      norm[i] = density_and_norm[2*i+1];
   }
   value = kT;
   kT = Double_Type[nfit];
   kT[*] = value;

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = ncomp_plasma(modelname,spectrum_type,_apec_stuff_->linelist,
			   kT,density,norm,abundances,redshift,
			   keV,binlo,binhi);

   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec_n () {

   % input variables 
   variable nfit;                                                  % 1
   variable kT, density, norm;                                     % 100+100+100
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 321
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable kT_and_density_and_norm, ncomp = 100;
   variable i, nargs = 21+3*ncomp, inc, nbin, integrated, value;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      kT_and_density_and_norm = array_from_args(__pop_args(3*ncomp));
      (nfit) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      kT_and_density_and_norm = array_from_args(__pop_args(3*ncomp));
      (nfit) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

   if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   %
   % Fix kT and Norm arrays
   %
   kT = Double_Type[nfit];
   density = Double_Type[nfit];
   norm = Double_Type[nfit];
   for(i=0;i<nfit;i++) {
      kT[i] = kT_and_density_and_norm[3*i];
      density[i] = kT_and_density_and_norm[3*i+1];
      norm[i] = kT_and_density_and_norm[3*i+2];
   }

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = ncomp_plasma(modelname,spectrum_type,_apec_stuff_->linelist,
			   kT,density,norm,abundances,redshift,
			   keV,binlo,binhi);

   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% Register the different models
%
%
define setup_apec_models () {
    variable ncomp = 100;
    variable nfit_name = "nfit", 
             nfit_default = 1, 
             nfit_min = 1, 
             nfit_max=ncomp, 
             nfit_thaw = 0;
    variable kT_name = "kT", 
             kT_default = 1.0, 
             kT_min = 0.0001, 
             kT_max=80.0, 
             kT_thaw = 1;
    variable density_name = "density", 
             density_default = 1.0, 
             density_min = -4.0, 
             density_max= 20.0, 
             density_thaw = 0;
    variable norm_name = "norm", 
             norm_default = 1.0, 
             norm_min = 0.0, 
             norm_max= 100.0, 
             norm_thaw = 1;
    variable kT_default_nfit = kT_default, 
             kT_min_nfit = kT_min, 
             kT_max_nfit = kT_max, 
             kT_thaw_nfit = 0;
    variable density_default_nfit = density_default, 
             density_min_nfit = density_min, 
             density_max_nfit = density_max, 
             density_thaw_nfit = 0;
    variable norm_default_nfit = 0.0, 
             norm_min_nfit = norm_min, 
             norm_max_nfit = norm_max, 
             norm_thaw_nfit = 0;
    variable elements = ["He","C","N","O","F","Ne","Mg",
                         "Al","Si","S","Ar","Ca","Fe","Ni"];
    variable abundance_name = "abundance", 
             abundance_default = 1.0, 
             abundance_min = 0.0, 
             abundance_max=100.0, 
             abundance_thaw = 0;
    variable redshift_name = "redshift", 
             redshift_default = 0.0, 
             redshift_min = 0.0, 
             redshift_max=10.0, 
             redshift_thaw = 0;
    variable nstring = 3;
    variable modelname_name = "modelname", 
             modelname_default = "apec";
    variable spectrum_type_name = "spectrum_type", 
             spectrum_type_default = "lines_and_continuum";
    variable linelist_name = "linelist", 
             linelist_default = "all";
    variable par_name, par_default, par_min, par_max, par_thaw, par_string;
    variable i, j, narg;

    use_namespace("_ciao");

    % For the apec_simple model
    par_name    = [kT_name,density_name,norm_name,abundance_name,redshift_name,
                   modelname_name,spectrum_type_name,linelist_name];
    par_default = [kT_default,density_default,norm_default,abundance_default,redshift_default];
    par_min     = [kT_min,density_min,norm_min,abundance_min,redshift_min];
    par_max     = [kT_max,density_max,norm_max,abundance_max,redshift_max];
    par_thaw    = [kT_thaw,density_thaw,norm_thaw,abundance_thaw,redshift_thaw];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];


    () = register_model("apec_simple",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % For apec model;
    narg = 21;
    par_name    = String_Type[narg];
    par_default = Double_Type[narg-nstring];
    par_min     = Double_Type[narg-nstring];
    par_max     = Double_Type[narg-nstring];
    par_thaw    = Integer_Type[narg-nstring];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];
    
    par_name[[0:2]]    = [kT_name,density_name,norm_name];
    par_default[[0:2]] = [kT_default,density_default,norm_default];
    par_min[[0:2]]     = [kT_min,density_min,norm_min];
    par_max[[0:2]]     = [kT_max,density_max,norm_max];
    par_thaw[[0:2]]    = [kT_thaw,density_thaw,norm_thaw];

    par_name[narg-nstring-1]    = redshift_name;
    par_default[narg-nstring-1] = redshift_default;
    par_min[narg-nstring-1]     = redshift_min;
    par_max[narg-nstring-1]     = redshift_max;
    par_thaw[narg-nstring-1]    = redshift_thaw;

    for (i=3;i<narg-nstring-1;i++) {
       par_name[i] = abundance_name+"_"+elements[i-3];
       par_default[i] = abundance_default;
       par_min[i]     = abundance_min;
       par_max[i]     = abundance_max;
       par_thaw[i]    = abundance_thaw;
    }

    % Temporary fixes
    variable par_string_cont = [modelname_default,"continuum",linelist_default];
    variable par_string_line = [modelname_default,"lines",linelist_default];

    par_name[[narg-nstring:narg-1]] = [modelname_name,spectrum_type_name,linelist_name];
      
    () = register_model("apec",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

    % Temporary fixes
    () = register_model("apec_cont",par_name,1,par_default,par_min,par_max,par_thaw,par_string_cont);

    () = register_model("apec_line",par_name,1,par_default,par_min,par_max,par_thaw,par_string_line);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % For the apec_nT model
    narg = 20+2*ncomp;
    par_name    = String_Type[narg];
    par_default = Double_Type[narg-nstring];
    par_min     = Double_Type[narg-nstring];
    par_max     = Double_Type[narg-nstring];
    par_thaw    = Integer_Type[narg-nstring];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];
    
    par_name[[0:1]]    = [nfit_name,density_name];
    par_default[[0:1]] = [nfit_default,density_default];
    par_min[[0:1]]     = [nfit_min,density_min];
    par_max[[0:1]]     = [nfit_max,density_max];
    par_thaw[[0:1]]    = [nfit_thaw,density_thaw];

    par_name[narg-nstring-1]    = redshift_name;
    par_default[narg-nstring-1] = redshift_default;
    par_min[narg-nstring-1]     = redshift_min;
    par_max[narg-nstring-1]     = redshift_max;
    par_thaw[narg-nstring-1]    = redshift_thaw;

    for (i=0;i<ncomp;i++) {
       par_name[2+2*i] = kT_name+string(i+1);
       par_name[2+2*i+1] = norm_name+string(i+1);
       if (i == 0) {
	  par_default[2+2*i] = kT_default;
	  par_default[2+2*i+1] = norm_default;
	  par_min[2+2*i] = kT_min;
	  par_min[2+2*i+1] = norm_min;
	  par_max[2+2*i] = kT_max;
	  par_max[2+2*i+1] = norm_max;
	  par_thaw[2+2*i] = kT_thaw;
	  par_thaw[2+2*i+1] = norm_thaw;
       } else {
	  par_default[2+2*i] = kT_default_nfit;
	  par_default[2+2*i+1] = norm_default_nfit;
	  par_min[2+2*i] = kT_min_nfit;
	  par_min[2+2*i+1] = norm_min_nfit;
	  par_max[2+2*i] = kT_max_nfit;
	  par_max[2+2*i+1] = norm_max_nfit;
	  par_thaw[2+2*i] = kT_thaw_nfit;
	  par_thaw[2+2*i+1] = norm_thaw_nfit;
       }
    }

    j = 0;
    for (i=2+2*ncomp;i<narg-nstring-1;i++) {
       par_name[i]    = abundance_name+"_"+elements[j];
       par_default[i] = abundance_default;
       par_min[i]     = abundance_min;
       par_max[i]     = abundance_max;
       par_thaw[i]    = abundance_thaw;
       j++;
    }
      
    par_name[[narg-nstring:narg-1]] = [modelname_name,spectrum_type_name,linelist_name];


    () = register_model("apec_nt",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % For the apec_ndensity model
    narg = 20+2*ncomp;
    par_name    = String_Type[narg];
    par_default = Double_Type[narg-nstring];
    par_min     = Double_Type[narg-nstring];
    par_max     = Double_Type[narg-nstring];
    par_thaw    = Integer_Type[narg-nstring];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];
    
    par_name[[0:1]]    = [nfit_name,kT_name];
    par_default[[0:1]] = [nfit_default,kT_default];
    par_min[[0:1]]     = [nfit_min,kT_min];
    par_max[[0:1]]     = [nfit_max,kT_max];
    par_thaw[[0:1]]    = [nfit_thaw,kT_thaw];

    par_name[narg-nstring-1]    = redshift_name;
    par_default[narg-nstring-1] = redshift_default;
    par_min[narg-nstring-1]     = redshift_min;
    par_max[narg-nstring-1]     = redshift_max;
    par_thaw[narg-nstring-1]    = redshift_thaw;

    for (i=0;i<ncomp;i++) {
       par_name[2+2*i] = density_name+string(i+1);
       par_name[2+2*i+1] = norm_name+string(i+1);
       if (i == 0) {
	  par_default[2+2*i] = density_default;
	  par_default[2+2*i+1] = norm_default;
	  par_min[2+2*i] = density_min;
	  par_min[2+2*i+1] = norm_min;
	  par_max[2+2*i] = density_max;
	  par_max[2+2*i+1] = norm_max;
	  par_thaw[2+2*i] = density_thaw;
	  par_thaw[2+2*i+1] = norm_thaw;
       } else {
	  par_default[2+2*i] = density_default_nfit;
	  par_default[2+2*i+1] = norm_default_nfit;
	  par_min[2+2*i] = density_min_nfit;
	  par_min[2+2*i+1] = norm_min_nfit;
	  par_max[2+2*i] = density_max_nfit;
	  par_max[2+2*i+1] = norm_max_nfit;
	  par_thaw[2+2*i] = density_thaw_nfit;
	  par_thaw[2+2*i+1] = norm_thaw_nfit;
       }
    }

    j = 0;
    for (i=2+2*ncomp;i<narg-nstring-1;i++) {
       par_name[i]    = abundance_name+"_"+elements[j];
       par_default[i] = abundance_default;
       par_min[i]     = abundance_min;
       par_max[i]     = abundance_max;
       par_thaw[i]    = abundance_thaw;
       j++;
    }
      
    par_name[[narg-nstring:narg-1]] = [modelname_name,spectrum_type_name,linelist_name];


    () = register_model("apec_ndenisty",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % For the apec_n model
     narg = 19+3*ncomp;
     par_name    = String_Type[narg];
     par_default = Double_Type[narg-nstring];
     par_min     = Double_Type[narg-nstring];
     par_max     = Double_Type[narg-nstring];
     par_thaw    = Integer_Type[narg-nstring];
     par_string  = [modelname_default,spectrum_type_default,linelist_default];
    
     par_name[0]    = nfit_name;
     par_default[0] = nfit_default;
     par_min[0]     = nfit_min;
     par_max[0]     = nfit_max;
     par_thaw[0]    = nfit_thaw;

     par_name[narg-nstring-1]    = redshift_name;
     par_default[narg-nstring-1] = redshift_default;
     par_min[narg-nstring-1]     = redshift_min;
     par_max[narg-nstring-1]     = redshift_max;
     par_thaw[narg-nstring-1]    = redshift_thaw;

     for (i=0;i<ncomp;i++) {
        par_name[1+3*i] = kT_name+string(i+1);
        par_name[1+3*i+1] = density_name+string(i+1);
        par_name[1+3*i+2] = norm_name+string(i+1);
        if (i == 0) {
 	  par_default[1+3*i] = kT_default;
 	  par_default[1+3*i+1] = density_default;
 	  par_default[1+3*i+2] = norm_default;
 	  par_min[1+3*i] = kT_min;
 	  par_min[1+3*i+1] = density_min;
 	  par_min[1+3*i+2] = norm_min;
 	  par_max[1+3*i] = kT_max;
 	  par_max[1+3*i+1] = density_max;
 	  par_max[1+3*i+2] = norm_max;
 	  par_thaw[1+3*i] = kT_thaw;
 	  par_thaw[1+3*i+1] = density_thaw;
 	  par_thaw[1+3*i+2] = norm_thaw;
        } else {
 	  par_default[1+3*i] = kT_default_nfit;
 	  par_default[1+3*i+1] = density_default_nfit;
 	  par_default[1+3*i+2] = norm_default_nfit;
 	  par_min[1+3*i] = kT_min_nfit;
 	  par_min[1+3*i+1] = density_min_nfit;
 	  par_min[1+3*i+2] = norm_min_nfit;
 	  par_max[1+3*i] = kT_max_nfit;
 	  par_max[1+3*i+1] = density_max_nfit;
 	  par_max[1+3*i+2] = norm_max_nfit;
 	  par_thaw[1+3*i] = kT_thaw_nfit;
 	  par_thaw[1+3*i+1] = density_thaw_nfit;
 	  par_thaw[1+3*i+2] = norm_thaw_nfit;
        }
     }

     j = 0;
     for (i=1+3*ncomp;i<narg-nstring-1;i++) {
        par_name[i]    = abundance_name+"_"+elements[j];
        par_default[i] = abundance_default;
        par_min[i]     = abundance_min;
        par_max[i]     = abundance_max;
        par_thaw[i]    = abundance_thaw;
        j++;
     }
      
     par_name[[narg-nstring:narg-1]] = [modelname_name,spectrum_type_name,linelist_name];


     () = register_model("apec_n",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

 }
