%
%
%
%
variable n_arfm = 0, n_arfp = 0;
variable arfm = Assoc_Type[];
variable arfp = Assoc_Type[];
variable fit_initialize = Assoc_Type[];

%
%
%
define reset_fit_initialize () {

   variable key;
   if (length(fit_initialize) != 0) {
      foreach (assoc_get_keys(fit_initialize)) {
	 key = ();
	 assoc_delete_key(fit_initialize,key);
      }
   }

}

%
% Read arfs and to make life simple redefine the higher order
% arfs to be on the same wavelengthscale as the first order arf.
%
define read_arfs(files_m_order,files_p_order) {

   variable i, n_m = length(files_m_order);
   variable n_p = length(files_p_order);
   variable key, arf1, arf, indx;
   
   n_arfm = n_m;
   n_arfp = n_p;
   for (i=0;i<n_m;i++) {
      key = string(i+1);
      arf =  readarf(expand_env(files_m_order[i]));
      if (i == 0) {
	 arf1 = arf;
	 indx = where(arf.SPECRESP == 0.0);
	 arf.SPECRESP[indx] = 1.0e-6;	     
      } else {
	 % Rebinning is not necessary, we can assume that 
	 % bin 1 of order 1 corresponds to the same point in the
	 % spectrum as bin 1 of order 2
	 arf.SPECRESP = arf.SPECRESP / arf1.SPECRESP;

      }
      arfm[key] = arf;
   }
   for (i=0;i<n_p;i++) {
      key = string(i+1);
      arf =  readarf(expand_env(files_p_order[i]));
      if (i == 0) {
	 arf1 = arf;
	 indx = where(arf.SPECRESP == 0.0);
	 arf.SPECRESP[indx] = 1.0e-6;	     
      } else {
	 % Rebinning is not necessary, we can assume that 
	 % bin 1 of order 1 corresponds to the same point in the
	 % spectrum as bin 1 of order 2

	 arf.SPECRESP = arf.SPECRESP / arf1.SPECRESP;

      }
      arfp[key] = arf;
   }

}

%
%
%
define get_arf_indices(binlo) {

   variable i, iii, indx = Integer_Type[length(binlo)], bl, arf;

   arf = arfm["1"];
   i = 0;
   % first find the right 
   foreach (binlo) {
      bl = ();
      iii = where(abs(arf.BIN_LO-bl) == min(abs(arf.BIN_LO-bl)));
      indx[i] = iii[0];
      i = i+1;
   }

   return(indx);

}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%


% Temporary fixes
define apec_cont_letg() {
   
   variable nargs = 23;
   variable arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
            arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
            arg21,arg22, arg23;
   variable spectrum;

   if (_NARGS == nargs-1) {
       usage("apec_cont_letg model cannot be used in integrate off mode");
      _pop_n(_NARGS);
      return NULL;
   } else if (_NARGS == nargs) {
      (arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,
       arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,
       arg21,arg22,arg23) = ();

      variable indx;
      variable dataset = get_dataset_number();
      variable arfs, arf, order, norder, i;

      if (0 == assoc_key_exists(fit_initialize,string(dataset))) {
	 indx = get_arf_indices(arg22);
	 fit_initialize[string(dataset)] = indx;
      } else {
	 indx = fit_initialize[string(dataset)];
      }

      if (dataset == 1) {
	 arfs = arfm;
	 norder = n_arfm;
      } else {
	 arfs = arfp;
	 norder = n_arfp;
      }
      
      for (i=0;i<norder;i++) {
	 order = string(i+1);
	 arf = arfs[order];
	 if (i == 0 ) {

	    spectrum = apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,
			    arg9,arg10,arg11,arg12,arg13,arg14,arg15,
			    arg16,arg17,arg18,arg19,arg20,arg21,
			    typecast(arf.BIN_LO[indx],_typeof(arg22)),
                            typecast(arf.BIN_HI[indx],_typeof(arg22)));
	 } else {
	    spectrum = spectrum + arf.SPECRESP[indx] * 
		apec(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,
			    arg9,arg10,arg11,arg12,arg13,arg14,arg15,
			    arg16,arg17,arg18,arg19,arg20,arg21,
			    typecast(arf.BIN_LO[indx],_typeof(arg22)),
                            typecast(arf.BIN_HI[indx],_typeof(arg22)));
	 }
      }
   } else {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   return(spectrum);
}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% Register the different models
%
%
define setup_apec_letg_models () {
    variable ncomp = 100;
    variable nfit_name = "nfit", 
             nfit_default = 1, 
             nfit_min = 1, 
             nfit_max=ncomp, 
             nfit_thaw = 0;
    variable kT_name = "kT", 
             kT_default = 1.0, 
             kT_min = 0.0001, 
             kT_max=80.0, 
             kT_thaw = 1;
    variable density_name = "density", 
             density_default = 1.0, 
             density_min = -4.0, 
             density_max= 20.0, 
             density_thaw = 0;
    variable norm_name = "norm", 
             norm_default = 1.0, 
             norm_min = 0.0, 
             norm_max= 100.0, 
             norm_thaw = 1;
    variable kT_default_nfit = kT_default, 
             kT_min_nfit = kT_min, 
             kT_max_nfit = kT_max, 
             kT_thaw_nfit = 0;
    variable density_default_nfit = density_default, 
             density_min_nfit = density_min, 
             density_max_nfit = density_max, 
             density_thaw_nfit = 0;
    variable norm_default_nfit = 0.0, 
             norm_min_nfit = norm_min, 
             norm_max_nfit = norm_max, 
             norm_thaw_nfit = 0;
    variable elements = ["He","C","N","O","F","Ne","Mg",
                         "Al","Si","S","Ar","Ca","Fe","Ni"];
    variable abundance_name = "abundance", 
             abundance_default = 1.0, 
             abundance_min = 0.0, 
             abundance_max=100.0, 
             abundance_thaw = 0;
    variable redshift_name = "redshift", 
             redshift_default = 0.0, 
             redshift_min = 0.0, 
             redshift_max=10.0, 
             redshift_thaw = 0;
    variable nstring = 3;
    variable modelname_name = "modelname", 
             modelname_default = "apec";
    variable spectrum_type_name = "spectrum_type", 
             spectrum_type_default = "lines_and_continuum";
    variable linelist_name = "linelist", 
             linelist_default = "all";
    variable par_name, par_default, par_min, par_max, par_thaw, par_string;
    variable i, j, narg;

    use_namespace("_ciao");

    % For the apec_simple model

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % For apec model;
    narg = 21;
    par_name    = String_Type[narg];
    par_default = Double_Type[narg-nstring];
    par_min     = Double_Type[narg-nstring];
    par_max     = Double_Type[narg-nstring];
    par_thaw    = Integer_Type[narg-nstring];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];
    
    par_name[[0:2]]    = [kT_name,density_name,norm_name];
    par_default[[0:2]] = [kT_default,density_default,norm_default];
    par_min[[0:2]]     = [kT_min,density_min,norm_min];
    par_max[[0:2]]     = [kT_max,density_max,norm_max];
    par_thaw[[0:2]]    = [kT_thaw,density_thaw,norm_thaw];

    par_name[narg-nstring-1]    = redshift_name;
    par_default[narg-nstring-1] = redshift_default;
    par_min[narg-nstring-1]     = redshift_min;
    par_max[narg-nstring-1]     = redshift_max;
    par_thaw[narg-nstring-1]    = redshift_thaw;

    for (i=3;i<narg-nstring-1;i++) {
       par_name[i] = abundance_name+"_"+elements[i-3];
       par_default[i] = abundance_default;
       par_min[i]     = abundance_min;
       par_max[i]     = abundance_max;
       par_thaw[i]    = abundance_thaw;
    }

    % Temporary fixes
    variable par_string_cont = [modelname_default,"continuum",linelist_default];
    variable par_string_line = [modelname_default,"lines",linelist_default];

    par_name[[narg-nstring:narg-1]] = [modelname_name,spectrum_type_name,linelist_name];
      
    %() = register_model("apec",par_name,1,par_default,par_min,par_max,par_thaw,par_string);

    % Temporary fixes
    () = register_model("apec_cont_letg",par_name,1,par_default,par_min,par_max,par_thaw,par_string_cont);

    %() = register_model("apec_line",par_name,1,par_default,par_min,par_max,par_thaw,par_string_line);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 }
