%
%
%
%
% Ronnie Hoogerwerf, SAO, 01/07/2004
%
use_namespace("LF");

define read_data() {

   %
   % Read in atomdb file. Adapted from "Guide" S-Lang script.
   %

   variable usage = "Usage: LF->read_data(filename [, alternative])";
   variable file = NULL, oldfile = NULL, gzfile = NULL, oldgzfile = NULL;

   switch (_NARGS) 
   { case 1 : file = (); }
   { case 2 : (file, oldfile) = (); }
   { _pop_n(_NARGS); message(usage); }

   % first get rid of any environmental variables
   file = LF_IO->replace_envs(file);
   gzfile = file + ".gz";

   !if (LF_IO->file_exists(file)) {  
      % straight filename doesnt work
      print("hello");
      if (LF_IO->file_exists(gzfile)) {  
	 % compressed does
	 file = gzfile;
      } else {
	 if (_NARGS == 2) {  
	    % are there two possibilities?
	    oldfile = LF_IO->replace_envs(oldfile);
	    oldgzfile = oldfile + ".gz";
	    if (LF_IO->file_exists(oldfile)) {
	       file = oldfile;
	    } else {
	       if (LF_IO->file_exists(oldgzfile)) {
		  file = oldgzfile;
	       } else {  
		  % two files, cant find either.
		  vmessage("Unable to read file %s or %s, even tried .gz", 
			   file, oldfile);
		  return NULL;		  
	       }
	    }
	 } else {  
	    % only one file
	    vmessage("Unable to read file %s, even tried %s.gz", file,file);
	    return NULL;
	 }
      }
   }


   %
   % get version number
   %
   variable fp = fopen(expand_env("$ATOMDB/VERSION"), "r");
   variable atomdbVer;
   if (fgets(&atomdbVer, fp) == -1) atomdbVer = "Unknown";
   () = fclose(fp);
   vmessage("==> Reading ATOMDB v%s to calculate line-free regions",atomdbVer[[:-2]]);

   % 
   %  At this point, filename is set to the correct filename for the line list.
   %  Must read the second HDU in the file.
   %
   variable DMfile = Sprintf("%s[2]",file,1);
   %
   %  Set varmm.caseinsen to 1 so that structure comes out lowercase.
   %
   variable old_caseinsen = Global->varmm.caseinsen;
   Global->varmm.caseinsen = 1;
   variable result = readfile(DMfile);
   %
   % WARNING: this is a temporary fix of the apec output. The emissivity of
   % the 21-15 transition for singly ionized Hydrogen (entry 33698 in the
   % file "apec_v1.10_linelist.fits"). The temporary fix is to substitute
   % the infinite emissivity with the one of the previous entry (also a
   % singly ionized Hydrogen transition).
   %
   result.emissivity[33697,0] = result.emissivity[33696,7];
   %
   %
   Global->varmm.caseinsen = old_caseinsen;
   LF->filename = file;
   LF->data = result; 
   
   variable i, n = result._nrows;
   variable wvl_temp = Double_Type[n,LF->n], peak, min = 0, max;

   for (i = 0; i < n; i++) {
      peak = result.peakindex[i] - 1;
      max = result.arraysize[i] - 1;
      wvl_temp[i,LF->iLambda] = result.lambda[i];            % Wavelength
      wvl_temp[i,LF->iEpeak] = result.emissivity[i,peak];   % Peak emmisivity
      wvl_temp[i,LF->iTpeak] = result.temperature[i,peak];  % Peak temperature
      wvl_temp[i,LF->iIon] = LF->get_ionbal(wvl_temp[i,LF->iTpeak],
					    result.element[i],
					    result.ion[i]);
      %wvl_temp[i,LF->iTmin] = result.temperature[i,min];   % Min temperature
      %wvl_temp[i,LF->iTmax] = result.temperature[i,max];   % Max temperature
      wvl_temp[i,LF->iTmin] = result.temperature[i,min] - 
	  (result.temperature[i,min+1] - result.temperature[i,min]);
      wvl_temp[i,LF->iTmax] = result.temperature[i,max] +
	  (result.temperature[i,max] - result.temperature[i,max-1]);
   }

   LF->data_short = wvl_temp;
   %LF->Tmin = LF_Math->min(wvl_temp[*,LF->iTmin]);
   %LF->Tmax = LF_Math->max(wvl_temp[*,LF->iTmax]);
   LF->Lmin = LF_Math->min(wvl_temp[*,LF->iLambda]);
   LF->Lmax = LF_Math->max(wvl_temp[*,LF->iLambda]);
   LF->epsmin = LF_Math->min(wvl_temp[*,LF->iEpeak]);
   LF->epsmax = LF_Math->max(wvl_temp[*,LF->iEpeak]);

   return 0;

}

