#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "descript.h"
/*
  Author: Randall Smith
  Date:  12/21/93
  Version: 1.1
  Language: ansi c
  Routine: read_parm_file
  Invocation: read_parm_file( char *inputfile, struct descript **des)
  Purpose: 
    This routine loads in all the parameters needed to do a run from a 
    text file.  The information must be given in the following
    format:

    DESCRIPTOR1   VALUE1 <return>

    DESCRIPTOR2   VALUE2 <return>

    and so on.  The results are returned in a linked list containing the
    descriptors and values (in a union). Input is taken from standard
    input.  In order to get the format of the number, the first character
    of the descriptor must be:
    I : integer,
    F : floating point,
    C : character (only one character allowed in value),
    S : string (result is a pointer; maxlength is MAXSTRLEN ),
    D : double  

    If the first letter is not one of these, the value will be taken to be
    integer 0, no matter value is given. 
  Revisions: None
  Side Effects: None
  Other Notes: None
  End:
*/

void read_parm_file( char *inputfile, struct descript **des) {
  
  FILE *fp;
  char format,*line;
  int result;
  struct descript *ptr=NULL;
  
  line = (char *) malloc(MAXSTRLEN);
  *des = NULL;
  if (inputfile != NULL) { 
    fp = fopen(inputfile,"r"); 
  } else {
    fp = stdin;
  }
  if ((long) fp ==  NULL) {
    printf("read_parm_file: Can't open %s\n", inputfile);
    abort();
  }
  while (fgets(line,MAXSTRLEN-1,fp) != NULL) {
    if (strlen(line) > 1 ) {
      
      /* Allocate some memory */
      
      if (*des==NULL) {
	ptr = (struct descript *) malloc(sizeof(struct descript));
	*des = ptr;
      } else {
	ptr->next = (struct descript *) malloc(sizeof(struct descript));
	ptr=ptr->next;
      }
      ptr->descriptor = (char *) malloc(MAXSTRLEN);
      ptr->next = NULL;
      
      /* Read the first character of the line */
      
      sscanf(line,"%c",&format);
      switch (format) {
      case 'I' :
	result = sscanf(line,"%s %d",(ptr->descriptor),&(ptr->value.ival) );
	break;
      case 'F' :
	result = sscanf(line,"%s %e",(ptr->descriptor),&(ptr->value.fval) );
	break;
      case 'C' :
	result = sscanf(line,"%s %c",(ptr->descriptor),&(ptr->value.cval) );
	break;
      case 'S' :
	ptr->value.sval = (char *) malloc(MAXSTRLEN);
	result = sscanf(line,"%s %s",(ptr->descriptor),ptr->value.sval );
	break;
      case 'D' :
	result = sscanf(line,"%s %le",(ptr->descriptor),&(ptr->value.dval));
	break;
      default:
	result = sscanf(line,"%s", (ptr->descriptor));
	ptr->value.ival = 0;
	break;
      }
      if (result == EOF) {
	printf("read_parm_file: Can't parse line:\n");
	printf("%s\n", line);
        abort();
      }       
    }
  }
}
