#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>

#include "slang.h"

#define TRUE 1
#define FALSE 0

static unsigned char _initialized;

int init_mie_module(void);

void miev0_(float *xx, float crefin[], int *perfct, float *mimcut, 
	    int *anyang, int *numang, float *xmu, 
	    int *nmom, int *ipolzn, int *momdim, 
	    int prnt[], float *qext, float *qsca, float *gqsc, 
	    float *pmom, float sforw[], float sback[], float s1[], 
	    float s2[], float tforw[], float tback[], float *spike);

void miev0_wrap(double xx, double crefin[2], int perfct, double mimcut, 
		int anyang, int numang, double *xmu,
		int nmom, int ipolzn, int momdim, int prnt[2], 
		double *qext, double *qsca, double *gqsc,
		double *pmom, double sforw[2], double sback[2], 
		double *s1, double *s2, double tforw[4], double tback[4],
		double *spike);
void miev0_slang();

static SLang_Intrin_Fun_Type Mie_Intrinsics [] = 
{
  MAKE_INTRINSIC_0("mie", miev0_slang, SLANG_VOID_TYPE),
  SLANG_END_INTRIN_FUN_TABLE
};

int init_mie_module(void) {

  if (_initialized) return 0;

  _initialized = 1;
  if (-1 == SLadd_intrin_fun_table (Mie_Intrinsics, NULL)) {
    return -1;
  }

  return 0;
}

void miev0_wrap(double xx, double crefin[2], int perfct, double mimcut, 
		int anyang, int numang, double *xmu,
		int nmom, int ipolzn, int momdim, int prnt[2], 
		double *qext, double *qsca, double *gqsc,
		double *pmom, double sforw[2], double sback[2], 
		double *s1, double *s2, double tforw[4], double tback[4],
		double *spike) {

  int i;
  float fl_xx, fl_crefin[2], fl_mimcut, *fl_xmu;
  float fl_qext, fl_qsca, fl_gqsc, *fl_pmom, fl_sforw[2], fl_sback[2];
  float *fl_s1, *fl_s2, fl_tforw[4], fl_tback[4], fl_spike;
  
  fl_xx = (float) xx;
  fl_crefin[0] = (float) crefin[0];
  fl_crefin[1] = (float) crefin[1];
  fl_mimcut = (float) mimcut;
  fl_xmu = (float *) malloc(numang*sizeof(float));
  for (i=0;i<numang;i++) fl_xmu[i] = (float) xmu[i];

  fl_s1 = (float *) malloc(2*numang*sizeof(float));
  fl_s2 = (float *) malloc(2*numang*sizeof(float));
  fl_pmom = (float *) malloc(momdim*(1+nmom)*sizeof(float));

  /*      LOGICAL  ANYANG, PERFCT, PRNT(*)
	  INTEGER  IPOLZN, MOMDIM, NUMANG, NMOM
	  REAL     GQSC, MIMCUT, PMOM( 0:MOMDIM, * ), QEXT, QSCA, SPIKE,
	  &         XMU(*), XX
	  COMPLEX  CREFIN, SFORW, SBACK,S1(*),S2(*), TFORW(*), TBACK(*)
  */

  miev0_( &fl_xx, fl_crefin, &perfct, &fl_mimcut, &anyang, 
	  &numang, fl_xmu, &nmom, &ipolzn, &momdim, prnt, 
	  &fl_qext, &fl_qsca, &fl_gqsc, fl_pmom, fl_sforw, fl_sback, 
	  fl_s1, fl_s2, fl_tforw, fl_tback, &fl_spike );
  
  *qext = (double) fl_qext;
  *qsca = (double) fl_qsca;
  *gqsc = (double) fl_gqsc;
  for (i=0;i<momdim*(1+nmom);i++) pmom[i] = (double) fl_pmom[i];
  sforw[0] = (double) fl_sforw[0];
  sforw[1] = (double) fl_sforw[1];
  sback[0] = (double) fl_sback[0];
  sback[1] = (double) fl_sback[1];
  for (i=0;i<2*numang;i++) {
    s1[i] = (double) fl_s1[i];
    s2[i] = (double) fl_s2[i];
  }
  for (i=0;i<4;i++) {
    tforw[i] = (double) fl_tforw[i];
    tback[i] = (double) fl_tback[i];
  }
  *spike = (double) fl_spike;

  free(fl_xmu);
  free(fl_s1);
  free(fl_s2);
  free(fl_pmom);
}

void miev0_slang(void) {

  int i;
  /* Inputs */
  double xx, mimcut, crefin[2];
  int perfct, anyang, numang, nmom,ipolzn,momdim,prnt[2];
  double *xmu;
  SLang_Array_Type *sl_xmu = NULL;
  /* Outputs */
  double qext, qsca, gqsc;
  SLang_Array_Type *sl_sforw  = NULL;
  SLang_Array_Type *sl_sback  = NULL;
  SLang_Array_Type *sl_s1     = NULL;
  SLang_Array_Type *sl_s2     = NULL;
  SLang_Array_Type *sl_tforw  = NULL;
  SLang_Array_Type *sl_tback  = NULL;
  double *pmom, sforw[2], sback[2];
  double *s1, *s2, tforw[4], tback[4];
  double spike;

  /* Create structure to hold output results */
  char *field_names[10];
  unsigned char field_types[10];
  VOID_STAR field_values[10];
  int dims;

  if (SLang_Num_Function_Args != 3) {
    SLang_verror (SL_USAGE_ERROR, "Usage: mie(blah blah blah)");
    return;
  }
  
  /* define ipolzn, prnt, momdim ourselves */
  if( (SLang_pop_array_of_type(&sl_xmu, SLANG_DOUBLE_TYPE) == -1 ) ||
      (SLang_pop_complex(&(crefin[0]), &(crefin[1])) == -1 ) ||
      (SLang_pop_double(&xx, NULL, NULL) == -1 ) ) {
    return;
  }

  anyang = 1; /* Allow any angles to be input.  Slight slowdown. */
  perfct = 0; /* Do not use infinite refractive index case */
  mimcut = 1.e-38; /* Minimum value for imaginary index */
  nmom = 0;  /* Do not calculate pmom */
  ipolzn = 0;
  prnt[0] = 0;
  prnt[1] = 0;
  numang = sl_xmu->num_elements;
  xmu = (double *) malloc(numang*sizeof(double));
  for (i=0;i<numang;i++) SLang_get_array_element(sl_xmu, &i, &(xmu[i]));
  momdim = 10;
  pmom = (double *) malloc(momdim*(1+nmom)*sizeof(double));
  s1 = (double *) malloc(2*numang*sizeof(double));
  s2 = (double *) malloc(2*numang*sizeof(double));

  miev0_wrap(xx, crefin, perfct, mimcut, anyang, numang, xmu,
	     nmom, ipolzn, momdim, prnt, &qext, &qsca, &gqsc,
	     pmom, sforw, sback, s1, s2, tforw, tback, &spike);
  
   /* transfer output values to s-lang array */
  field_names[0] = "qext";  field_types[0] = SLANG_DOUBLE_TYPE;
  field_names[1] = "qsca";  field_types[1] = SLANG_DOUBLE_TYPE;
  field_names[2] = "gqsc";  field_types[2] = SLANG_DOUBLE_TYPE;
  field_names[3] = "sforw"; field_types[3] = SLANG_COMPLEX_TYPE;
  field_names[4] = "sback"; field_types[4] = SLANG_COMPLEX_TYPE;
  field_names[5] = "s1";    field_types[5] = SLANG_ARRAY_TYPE;
  field_names[6] = "s2";    field_types[6] = SLANG_ARRAY_TYPE;
  field_names[7] = "tforw"; field_types[7] = SLANG_ARRAY_TYPE;
  field_names[8] = "tback"; field_types[8] = SLANG_ARRAY_TYPE;

  field_values[0] = &qext;
  field_values[1] = &qsca;
  field_values[2] = &gqsc;
  field_values[3] = &sforw;  /* Experiment.  Should work, since declared */
  field_values[4] = &sback;  /* complex & in order.   */

  /* These are real arrays */
  sl_s1    = SLang_create_array(SLANG_COMPLEX_TYPE, 0, s1, &numang, 1);
  if ( sl_s1 == NULL ) { return ;}
  sl_s2    = SLang_create_array(SLANG_COMPLEX_TYPE, 0, s2, &numang, 1);
  if ( sl_s2 == NULL ) { return ;}
  dims = 2;
  sl_tforw = SLang_create_array(SLANG_COMPLEX_TYPE, 0, tforw, &dims, 1);
  if ( sl_tforw == NULL ) { return ;}
  sl_tback = SLang_create_array(SLANG_COMPLEX_TYPE, 0, tback, &dims, 1);
  if ( sl_tback == NULL ) { return ;}

  /*
  for (i=0;i<2*numang;i++) {
    SLang_set_array_element( sl_s1, &i, &(s1[i]) );
    SLang_set_array_element( sl_s2, &i, &(s2[i]) );
  }
  for (i=0;i<4;i++) {
    SLang_set_array_element( sl_tforw, &i, &(tforw[i]) );
    SLang_set_array_element( sl_tback, &i, &(tback[i]) );
  }
  */

  field_values[5] = &sl_s1;
  field_values[6] = &sl_s2;
  field_values[7] = &sl_tforw;
  field_values[8] = &sl_tback;

  if (SLstruct_create_struct(9, field_names, field_types, field_values)==-1){
    return;
  }

  free(xmu);
  free(pmom);
  free(s1);
  free(s2);

   return;
}

