/** \file TearOffMenu.h
    \brief Declaration of TearOffMenu class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_TearOffMenu_h
#define profit_TearOffMenu_h

#include "RootFrame.h"
#include "Rtypes.h"
#include "RQ_OBJECT.h"

class TGMainFrame;
class TGPopupMenu;

#include <map>
#include <string>

namespace profit {

  class Gui;

  /** \class TearOffMenu
      \brief Provides a torn-off copy of a pop-up menu. So far this is decoupled from the original menu, so if
             the menu changes, the torn-off copy is not updated.
  */
  class TearOffMenu {
    public:
      RQ_OBJECT("profit::TearOffMenu")
      ClassDef(TearOffMenu, 0)

      // Note: pointer to member function not used due to limitation of Cint.
      static TearOffMenu * display(const std::string & menu_id, Gui * gui, TGPopupMenu * popup_menu, int handler_id);

      virtual ~TearOffMenu();

      virtual void handleCloseWindow();

      virtual void handleMenu(int id);

      virtual TGPopupMenu * getMenu();

    protected:
      TearOffMenu(const std::string & menu_id, Gui * gui, TGPopupMenu * popup_menu);

      virtual void copy(const TGPopupMenu * menu);

      virtual void raise();

      virtual void setHandler(int handler_id);

    private:
      typedef std::map<std::string, TearOffMenu *> prototype_cont_type;
      static prototype_cont_type s_prototype;
      std::string m_menu_id;
      RootFrameTracker m_tracker;
      Gui * m_gui;
      TGMainFrame * m_main_frame;
      TGPopupMenu * m_popup_menu;
      int m_handler_id;
  };

}

#endif
