/** \file AtomDb.cxx
    \brief Implementation of AtomDb class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "ApedDatabase.h"
#include "AtomDbImp.h"
#include "XstarDatabase.h"
#include "profit/AtomDb.h"

#include <stdexcept>

namespace profit {

  AtomDb::AtomDb(const std::string & file_name): m_imp(0) {
    try {
      m_imp = new ApedDatabase(file_name);
    } catch (const std::exception &) {
      m_imp = new XstarDatabase(file_name);
    }
    if (0 == m_imp) throw std::runtime_error("AtomDb constructor could not allocate AtomDbImp object");
  }

  AtomDb::~AtomDb() { delete m_imp; }

  void AtomDb::filter(const IEnergy & min_energy, const IEnergy & max_energy) {
    m_imp->filter(min_energy, max_energy);
  }

  void AtomDb::filter(double min_emissivity) {
    m_imp->filter(min_emissivity);
  }

  const AtomDb::AtomicLineCont & AtomDb::getMatches() const {
    return m_imp->getMatches();
  }

}
