/** \file DataImport.cxx
    \brief Implementation of DataImport and related classes.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/DataImport.h"
#include "profit/Gui.h"

#include "TGButton.h"
#include "TGClient.h"
#include "TGComboBox.h"
#include "TGFrame.h"
#include "TGLabel.h"
#include "TGLayout.h"
#include "TGTextEntry.h"

#include <stdexcept>

#include <iostream>

ClassImp(profit::DataImport)

namespace profit {

  DataImport::DataImport(Gui * gui): m_tracker(), m_gui(gui) {
  }

  DataImport::~DataImport() {
    m_tracker.clear();
  }

  void DataImport::runInTestMode() {
    gClient->WaitFor(buildGui());
  }

  void DataImport::handleCloseWindow() {
    m_tracker.clear();
  }

  TGTransientFrame * DataImport::buildGui() {
    m_tracker.clear();

    TGTransientFrame * transient_frame = new TGTransientFrame(gClient->GetRoot(), m_gui->getRootFrame(), 600, 400);
    if (0 == transient_frame) throw std::runtime_error("DataImport::runInTestMode() could not create file import dialog box");
    transient_frame->Connect("CloseWindow", "profit::DataImport", this, "handleCloseWindow()");
    transient_frame->SetMinWidth(300);

    TGCompositeFrame * cf = new TGHorizontalFrame(transient_frame, 600, 22);
    m_tracker.trackFrame("Button frame", cf, 0, kLHintsExpandX);
    transient_frame->AddFrame(cf, m_tracker.back().getHints());

    TGButton * button = new TGTextButton(cf, "OK");
    m_tracker.trackFrame("OK button", button, cf, kLHintsNormal, 5, 0, 5, 0);

    button = new TGTextButton(cf, "Cancel");
    m_tracker.trackFrame("Cancel button", button, cf, kLHintsNormal, 0, 0, 5, 0);

#if 0
    m_interp_frame = new TGVerticalFrame(transient_frame, 10, 10);
    m_tracker.trackFrame("File interpreter container frame", m_interp_frame, 0, kLHintsExpandX | kLHintsExpandY);
    transient_frame->AddFrame(m_interp_frame, m_tracker.back().getHints());

    m_interpreter.push_back(new FileInterpreter(m_interp_frame));

    button = new TGTextButton(transient_frame, "Add File");
    m_tracker.trackFrame("Add file button", button, 0, kLHintsNormal, 5);
    transient_frame->AddFrame(button, m_tracker.back().getHints());
    button->Connect("Clicked", "profit::DataImport", this, "handleAddInterpreter()");
#endif

    transient_frame->MapSubwindows();
    transient_frame->Layout();
    transient_frame->MapWindow();

    return transient_frame;
  }

}
