/** \file Datum.cxx
    \brief Implementation of Datum class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/DataView.h"
#include "profit/Datum.h"

#include <stdexcept>

namespace profit {

  Datum::Datum(const std::string & name, const std::string & value): m_name(name), m_value(value), m_child(), m_view() {}

  std::string Datum::getName() const { return m_name; }

  std::string Datum::getValue() const { return m_value; }

  void Datum::setValue(const std::string & value) { m_value = value; }

  Datum & Datum::insertChild(const std::string & name, const std::string & value) {
    iterator itor = m_child.find(name);
    if (m_child.end() != itor)
      throw std::logic_error("Datum::insertChild: child already exists; cannot add child named \"" + name + "\"");
    else
      itor = m_child.insert(itor, value_type(name, Datum(name, value)));
    return itor->second;
  }

  Datum::iterator Datum::begin() { return m_child.begin(); }

  Datum::iterator Datum::end() { return m_child.end(); }

  Datum::const_iterator Datum::begin() const { return m_child.begin(); }

  Datum::const_iterator Datum::end() const { return m_child.end(); }

  void Datum::updateAllViews() {
    DataView::const_data_cont_type data_cont;
    data_cont.insert(this);
    for (view_cont_type::iterator itor = m_view.begin(); itor != m_view.end(); ++itor)
      (*itor)->update(data_cont);
  }

  void Datum::addView(DataView & view) {
    m_view.insert(&view);
  }

  void Datum::getAffectedViews(Datum::view_cont_type & view_cont) const {
    view_cont.insert(m_view.begin(), m_view.end());
  }

}
